@@ -8,8 +8,9 @@ array2list <- function(x) {
   n.dim <- length(dim(x))
   l <- list(length = dim(x)[n.dim])
   ind <- collapse(rep(",", n.dim - 1))
-  for (i in 1:dim(x)[n.dim])
-    l[[i]] <- eval(parse(text = paste0("x[", ind, i,"]")))
+  for (i in seq_len(dim(x)[n.dim])) {
+    l[[i]] <- eval(parse(text = paste0("x[", ind, i,"]"))) 
+  }
   names(l) <- dimnames(x)[[n.dim]]
   l
 }
@@ -35,14 +36,59 @@ Nsamples <- function(x, subset = NULL) {
   out
 }
 
-algorithm <- function(x) {
-  if (!is(x, "brmsfit")) {
-    stop("x must be of class brmsfit")
+contains_samples <- function(x) {
+  if (!is(x$fit, "stanfit") || !length(x$fit@sim)) {
+    stop("The model does not contain posterior samples",
+         call. = FALSE)
   }
+  invisible(TRUE)
+}
+
+algorithm <- function(x) {
+  stopifnot(is(x, "brmsfit"))
   if (is.null(x$algorithm)) "sampling"
   else x$algorithm
 }
 
+restructure <- function(x) {
+  # restructure old brmsfit objects to work with the latest brms version
+  stopifnot(is(x, "brmsfit"))
+  if (isTRUE(attr(x, "restructured"))) {
+    return(x)  # already restructured
+  }
+  # x$nonlinear deprecated as of brms > 0.9.1
+  # X$partial deprecated as of brms > 0.8.0
+  x$formula <- SW(update_formula(x$formula, partial = x$partial, 
+                                 nonlinear = x$nonlinear))
+  x$nonlinear <- x$partial <- NULL
+  if (is(x$autocor, "cor_fixed")) {
+    # deprecated as of brms 1.0.0
+    class(x$autocor) <- "cov_fixed"
+  }
+  if (is.null(x$version) || x$version <= "0.10.0.9000") {
+    attr(x$formula, "old_mv") <- is.old_mv(x)
+    ee <- extract_effects(formula(x), family = family(x))
+    x$ranef <- tidy_ranef(ee, model.frame(x))
+    if (length(ee$nonlinear)) {
+      # nlpar and group have changed positions
+      change <- change_old_ranef(x$ranef, pars = parnames(x),
+                                 dims = x$fit@sim$dims_oi)
+      for (i in seq_along(change)) {
+        x <- do_renaming(change = change[[i]], x = x)
+      }
+    }
+  } else if (x$version < "1.0.0") {
+    # I added double underscores in group-level parameters
+    # right before the release of brms 1.0.0
+    change <- change_old_ranef2(x$ranef, pars = parnames(x),
+                                dims = x$fit@sim$dims_oi)
+    for (i in seq_along(change)) {
+      x <- do_renaming(change = change[[i]], x = x)
+    }
+  }
+  structure(x, "restructured" = TRUE)
+}
+
 first_greater <- function(A, target, i = 1) {
   # find the first element in A that is greater than target
   # Args: 
@@ -67,6 +113,7 @@ link <- function(x, link) {
   else if (link == "inverse") 1/x
   else if (link == "sqrt") sqrt(x)
   else if (link == "1/mu^2") 1 / x^2
+  else if (link == "tan_half") tan(x / 2)
   else if (link == "logit") logit(x)
   else if (link == "probit") qnorm(x)
   else if (link == "probit_approx") qnorm(x)
@@ -87,6 +134,7 @@ ilink <- function(x, link) {
   else if (link == "inverse") 1/x
   else if (link == "sqrt") x^2
   else if (link == "1/mu^2") 1 / sqrt(x)
+  else if (link == "tan_half") 2 * atan(x)
   else if (link == "logit") inv_logit(x)
   else if (link == "probit") pnorm(x)
   else if (link == "probit_approx") inv_logit(0.07056*x^3 + 1.5976*x)
@@ -95,23 +143,29 @@ ilink <- function(x, link) {
   else stop(paste("Link", link, "not supported"))
 }
 
-get_cornames <- function(names, type = "cor", brackets = TRUE) {
+get_cornames <- function(names, type = "cor", brackets = TRUE,
+                         sep = "__") {
   # get correlation names as combinations of variable names
   # Args:
   #   names: the variable names 
-  #   type: of the correlation to be put in front of the returned strings
+  #   type: character string to be put in front of the returned strings
   #   brackets: should the correlation names contain brackets 
   #            or underscores as seperators
+  #   sep: character string to separate names;
+  #        only used if brackets == FALSE
   # Returns: 
-  #   correlation names based on the variable names passed to the names argument
+  #   correlation names based on the variable names 
+  #   passed to the names argument
   cornames <- NULL
   if (length(names) > 1) {
     for (i in 2:length(names)) {
       for (j in 1:(i-1)) {
         if (brackets) {
-          cornames <- c(cornames, paste0(type,"(",names[j],",",names[i],")"))
+          cornames <- c(cornames, 
+            paste0(type, "(", names[j], "," , names[i], ")"))
         } else {
-          cornames <- c(cornames, paste0(type,"_",names[j],"_",names[i]))
+          cornames <- c(cornames, 
+            paste0(type, sep, names[j], sep, names[i]))
         }
       }
     }
@@ -122,7 +176,7 @@ get_cornames <- function(names, type = "cor", brackets = TRUE) {
 get_nlpar <- function(x, suffix = "") {
   # extract name of a non-linear parameter
   nlpar <- attr(x, "nlpar")
-  if (!is.null(nlpar)) paste0(nlpar, suffix) else ""
+  if (length(nlpar) && nchar(nlpar)) paste0(nlpar, suffix) else ""
 }
 
 get_estimate <- function(coef, samples, margin = 2, to.array = FALSE, ...) {
@@ -170,18 +224,18 @@ get_summary <- function(samples, probs = c(0.025, 0.975),
   } else {
     coefs <- c("mean", "sd", "quantile")
   }
-  if (length(dim(samples)) == 2) {
+  if (length(dim(samples)) == 2L) {
     out <- do.call(cbind, lapply(coefs, get_estimate, samples = samples,
                                  probs = probs, na.rm = TRUE))
-  } else if (length(dim(samples)) == 3) {
+  } else if (length(dim(samples)) == 3L) {
     out <- abind(lapply(1:dim(samples)[3], function(i)
       do.call(cbind, lapply(coefs, get_estimate, samples = samples[, , i],
                             probs = probs))), along = 3)
     dimnames(out) <- list(NULL, NULL, paste0("P(Y = ", 1:dim(out)[3], ")")) 
   } else { 
-    stop("dimension of samples must be either 2 or 3") 
+    stop("dimension of 'samples' must be either 2 or 3") 
   }
-  rownames(out) <- 1:nrow(out)
+  rownames(out) <- seq_len(nrow(out))
   colnames(out) <- c("Estimate", "Est.Error", paste0(probs * 100, "%ile"))
   out  
 }
@@ -192,13 +246,15 @@ get_table <- function(samples, levels = sort(unique(as.numeric(samples)))) {
   #   samples: a S x N matrix
   #   levels: all possible values in \code{samples}
   # Returns:
-  #    a N x \code{levels} matrix containing absolute frequencies in each column seperately
-  if (!is.matrix(samples)) 
-    stop("samples must be a matrix")
-  out <- do.call(rbind, lapply(1:ncol(samples), function(n) 
-    table(factor(samples[, n], levels = levels))))
-  rownames(out) <- 1:nrow(out)
-  colnames(out) <- paste0("N(Y = ", 1:ncol(out), ")")
+  #    a N x \code{levels} matrix containing relative frequencies
+  #    in each column seperately
+  stopifnot(is.matrix(samples))
+  out <- do.call(rbind, lapply(seq_len(ncol(samples)), 
+    function(n) table(factor(samples[, n], levels = levels))))
+  # compute relative frequencies
+  out <- out / sum(out[1, ])
+  rownames(out) <- seq_len(nrow(out))
+  colnames(out) <- paste0("P(Y = ", seq_len(ncol(out)), ")")
   out
 }
 
@@ -212,28 +268,29 @@ get_cov_matrix <- function(sd, cor = NULL) {
   #   used in VarCorr.brmsfit
   # Returns: 
   #   samples of covariance and correlation matrices
-  if (any(sd < 0)) 
-    stop("standard deviations must be non negative")
+  sd <- as.matrix(sd)
+  stopifnot(all(sd >= 0))
   if (!is.null(cor)) {
-    if (ncol(cor) != ncol(sd) * (ncol(sd) - 1) / 2 || nrow(sd) != nrow(cor))
-      stop("dimensions of standard deviations and corrrelations do not match")
-    if (any(cor < -1 || cor > 1)) 
-      stop("correlations must be between -1 and 1")  
+    cor <- as.matrix(cor)
+    stopifnot(ncol(cor) == ncol(sd) * (ncol(sd) - 1) / 2, 
+              nrow(sd) == nrow(cor), min(cor) >= -1 && max(cor) <= 1)
   }
   nsamples <- nrow(sd)
   nranef <- ncol(sd)
-  cor_matrix <- aperm(array(diag(1, nranef), dim = c(nranef, nranef, nsamples)), 
-                      perm = c(3, 1, 2))
+  cor_matrix <- array(diag(1, nranef), dim = c(nranef, nranef, nsamples))
+  cor_matrix <- aperm(cor_matrix, perm = c(3, 1, 2))
   cov_matrix <- cor_matrix
-  for (i in 1:nranef) 
-    cov_matrix[, i, i] <- sd[, i]^2 
+  for (i in seq_len(nranef)) { 
+    cov_matrix[, i, i] <- sd[, i]^2
+  }
   if (!is.null(cor)) {
     k <- 0 
     for (i in 2:nranef) {
       for (j in 1:(i-1)) {
         k = k + 1
         cor_matrix[, j, i] <- cor_matrix[, i, j] <- cor[, k]
-        cov_matrix[, j, i] <- cov_matrix[, i, j] <- cor[, k] * sd[, i] * sd[, j]
+        cov_matrix[, j, i] <- 
+          cov_matrix[, i, j] <- cor[, k] * sd[, i] * sd[, j]
       }
     }
   }
@@ -253,17 +310,15 @@ get_cov_matrix_ar1 <- function(ar, sigma, nrows, se2 = 0) {
                perm = c(3, 1, 2))
   sigma2_adjusted <- sigma^2 / (1 - ar^2)
   pow_ar <- as.list(rep(1, nrows + 1))
-  for (i in 1:nrows) {
+  for (i in seq_len(nrows)) {
     pow_ar[[i + 1]] <- ar^i
     mat[, i, i] <- mat[, i, i] + sigma2_adjusted
-    if (i > 1) {
-      for (j in 1:(i - 1)) { 
-        mat[, i, j] <- sigma2_adjusted * pow_ar[[i - j + 1]]
-        mat[, j, i] <- mat[, i, j]
-      } 
-    }
+    for (j in seq_len(i - 1)) { 
+      mat[, i, j] <- sigma2_adjusted * pow_ar[[i - j + 1]]
+      mat[, j, i] <- mat[, i, j]
+    } 
   } 
-  mat 
+  mat
 }
 
 get_cov_matrix_ma1 <- function(ma, sigma, nrows, se2 = 0) {
@@ -280,7 +335,7 @@ get_cov_matrix_ma1 <- function(ma, sigma, nrows, se2 = 0) {
   sigma2 <- sigma^2
   sigma2_adjusted <- sigma2 * (1 + ma^2)
   sigma2_times_ma <- sigma2 * ma
-  for (i in 1:nrows) { 
+  for (i in seq_len(nrows)) { 
     mat[, i, i] <- mat[, i, i] + sigma2_adjusted
     if (i > 1) {
       mat[, i, i - 1] <- sigma2_times_ma
@@ -308,15 +363,13 @@ get_cov_matrix_arma1 <- function(ar, ma, sigma, nrows, se2 = 0) {
   gamma0 <- 1 + ma^2 + 2 * ar * ma
   gamma <- as.list(rep(NA, nrows))
   gamma[[1]] <- (1 + ar * ma) * (ar + ma)
-  for (i in 1:nrows) {
+  for (i in seq_len(nrows)) {
     mat[, i, i] <- mat[, i, i] + sigma2_adjusted * gamma0
     gamma[[i]] <- gamma[[1]] * ar^(i - 1)
-    if (i > 1) {
-      for (j in 1:(i - 1)) { 
-        mat[, i, j] <- sigma2_adjusted * gamma[[i - j]]
-        mat[, j, i] <- mat[, i, j]
-      } 
-    }
+    for (j in seq_len(i - 1)) { 
+      mat[, i, j] <- sigma2_adjusted * gamma[[i - j]]
+      mat[, j, i] <- mat[, i, j]
+    } 
   } 
   mat 
 }
@@ -333,79 +386,109 @@ get_cov_matrix_ident <- function(sigma, nrows, se2 = 0) {
   mat <- aperm(array(diag(se2, nrows), dim = c(nrows, nrows, nrow(sigma))),
                perm = c(3, 1, 2))
   sigma2 <- sigma^2
-  for (i in 1:nrows) {
+  for (i in seq_len(nrows)) {
     mat[, i, i] <- mat[, i, i] + sigma2
   }
   mat
 }
 
-get_sigma <- function(x, data, i, method = c("fitted", "predict", "logLik")) {
-  # get the residual standard devation of linear models
+get_auxpar <- function(x, i = NULL) {
+  # get samples of an auxiliary parameter
   # Args:
-  #   x: a brmsfit object or posterior samples of sigma (can be NULL)
+  #   x: object to extract postarior samples from
   #   data: data initially passed to Stan
-  #   method: S3 method from which get_sigma is called
-  #   i: meaning depends on the method argument:
-  #      for predict and logLik this is the current observation number
-  #      for fitted this is the number of samples
-  method <- match.arg(method)
-  if (is(x, "brmsfit")) {
-    sigma <- posterior_samples(x, pars = "^sigma($|_)")$sigma
+  #   i: the current observation number
+  #      (used in predict and logLik)
+  if (is.list(x)) {
+    # compute auxpar in distributional regression models
+    ilink <- get(x[["ilink"]], mode = "function")
+    x <- ilink(get_eta(i = if (!is.null(i)) i else NULL, draws = x))
   } else {
-    sigma <- x
-  }
-  sigma <- as.vector(sigma)
-  if (is.null(sigma)) {
-    # user defined standard errors were applied
-    sigma <- data$se
-    if (is.null(sigma)) {
-      # for backwards compatibility with brms <= 0.5.0
-      sigma <- data$sigma
-    }
-    if (is.null(sigma)) {
-      stop("no residual standard deviation(s) found")
-    }
-    if (method %in% c("predict", "logLik")) {
-      sigma <- sigma[i]
-    } else {
-      sigma <- matrix(rep(sigma, i), ncol = data$N, byrow = TRUE)
-    }
-  } else if (!is.null(data$disp)) {
-    if (method %in% c("predict", "logLik")) {
-      sigma <- sigma * data$disp[i]
-    } else {
-      # results in a Nsamples x Nobs matrix
-      sigma <- sigma %*% matrix(data$disp, nrow = 1)
+    if (!is.null(i) && isTRUE(ncol(x) > 1L)) {
+      x <- x[, i, drop = FALSE]
     }
   }
-  sigma
+  if (is.null(i) && isTRUE(ncol(x) == 1L)) {
+    # for compatibility with fitted helper functions
+    x <- as.vector(x)
+  }
+  x
 }
 
-get_shape <- function(x, data, i = NULL,
-                      method = c("fitted", "predict", "logLik")) {
-  # get the shape parameter of gamma, weibull and negbinomial models
-  # Args:
-  #   x: a brmsfit object or posterior samples of shape (can be NULL)
-  #   data: data initially passed to Stan
-  #   method: S3 method from which get_shape is called
-  #   i: only used for "predict" and "logLik": 
-  #      the current observation number
-  method <- match.arg(method)
-  if (is(x, "brmsfit")) {
-    shape <- posterior_samples(x, pars = "^shape$")$shape
+get_sigma <- function(x, data, i = NULL, dim = NULL) {
+  # get the residual standard devation of linear models
+  # Args: 
+  #    see get_auxpar
+  #    dim: target dimension of output matrices (used in fitted)
+  stopifnot(is.atomic(x) || is.list(x))
+  if (is.null(x)) {
+    x <- get_se(data = data, i = i, dim = dim)
   } else {
-    shape <- x
+    x <- get_auxpar(x, i = i)
   }
-  shape <- as.vector(shape)
+  mult_disp(x, data = data, i = i, dim = dim)
+}
+
+get_shape <- function(x, data, i = NULL, dim = NULL) {
+  # get the shape parameter of gamma, weibull and negbinomial models
+  # Args: see get_auxpar
+  stopifnot(is.atomic(x) || is.list(x))
+  x <- get_auxpar(x, i = i)
+  mult_disp(x, data = data, i = i, dim = dim)
+}
+
+get_theta <- function(draws, i = NULL, par = c("zi", "hu")) {
+  # convenience function to extract zi / hu parameters
+  # also works with deprecated models fitted with brms < 1.0.0 
+  # which were using multivariate syntax
+  # Args:
+  #   see get_auxpar
+  #   par: parameter to extract; either 'zi' or 'hu'
+  par <- match.arg(par)
+  if (!is.null(draws$data$N_trait)) {
+    j <- if (!is.null(i)) i else seq_len(draws$data$N_trait)
+    theta <- ilink(get_eta(draws, j + draws$data$N_trait), "logit")
+  } else {
+    theta <- get_auxpar(draws[[par]], i = i)
+  }
+  theta
+}
+
+get_se <- function(data, i = NULL, dim = NULL) {
+  # extract user-defined standard errors
+  # Args: see get_auxpar
+  se <- data$se
+  if (is.null(se)) {
+    # for backwards compatibility with brms <= 0.5.0
+    se <- data$sigma
+  }
+  if (!is.null(i)) {
+    se <- se[i]
+  } else {
+    stopifnot(!is.null(dim))
+    se <- matrix(se, nrow = dim[1], ncol = dim[2], byrow = TRUE)
+  }
+  se
+}
+
+mult_disp <- function(x, data, i = NULL, dim = NULL) {
+  # multiply existing samples by 'disp' data
+  # Args: see get_auxpar
   if (!is.null(data$disp)) {
-    if (method %in% c("predict", "logLik")) {
-      shape <- shape * data$disp[i]
+    if (!is.null(i)) {
+      x <- x * data$disp[i]
     } else {
       # results in a Nsamples x Nobs matrix
-      shape <- shape %*% matrix(data$disp, nrow = 1)
+      if (is.matrix(x)) {
+        stopifnot(!is.null(dim))
+        disp <- matrix(disp, nrow = dim[1], ncol = dim[2], byrow = TRUE)
+        x <- x * disp
+      } else {
+        x <- x %*% matrix(data$disp, nrow = 1) 
+      }
     }
   }
-  shape
+  x
 }
 
 prepare_family <- function(x) {
@@ -416,10 +499,10 @@ prepare_family <- function(x) {
   if (is.old_lognormal(family, nresp = nresp, version = x$version)) {
     family <- lognormal()
   } else if (is.linear(family) && nresp > 1L) {
-    family$family <- paste0(family$family, "_multi")
+    family$family <- paste0(family$family, "_mv")
   } else if (use_cov(x$autocor) && sum(x$autocor$p, x$autocor$q) > 0) {
     family$family <- paste0(family$family, "_cov")
-  } else if (is(x$autocor, "cor_fixed")) {
+  } else if (is(x$autocor, "cov_fixed")) {
     family$family <- paste0(family$family, "_fixed")
   }
   family
@@ -436,8 +519,9 @@ extract_pars <- function(pars, all_pars, exact_match = FALSE,
   #   ...: Further arguments to be passed to grepl
   # Returns:
   #   A character vector of parameter names
-  if (!(anyNA(pars) || is.character(pars))) 
+  if (!(anyNA(pars) || is.character(pars))) {
     stop("Argument pars must be NA or a character vector")
+  }
   if (!anyNA(pars)) {
     if (exact_match) {
       pars <- all_pars[all_pars %in% pars]
@@ -460,11 +544,7 @@ compute_ic <- function(x, ic = c("waic", "loo"), ll_args = list(), ...) {
   # Returns:
   #   output of the loo package with amended class attribute
   ic <- match.arg(ic)
-  if (!is(x, "brmsfit")) 
-    stop(paste("Cannot compute information criteria for", 
-               "an object of class", class(x)), call. = FALSE)
-  if (!is(x$fit, "stanfit") || !length(x$fit@sim)) 
-    stop("The model does not contain posterior samples")
+  contains_samples(x)
   args <- list(x = do.call(logLik, c(list(x), ll_args)))
   if (ll_args$pointwise) {
     args$args$draws <- attr(args$x, "draws")
@@ -495,7 +575,7 @@ compare_ic <- function(x, ic = c("waic", "loo")) {
   rnames <- rep("", nrow(ic_diffs))
   # pairwise comparision to get differences in ICs and their SEs
   n <- 1
-  for (i in 1:(n_models - 1)) {
+  for (i in seq_len(n_models - 1)) {
     for (j in (i + 1):n_models) {
       temp <- loo::compare(x[[j]], x[[i]])
       ic_diffs[n, ] <- c(-2 * temp[["elpd_diff"]], 2 * temp[["se"]]) 
@@ -507,7 +587,7 @@ compare_ic <- function(x, ic = c("waic", "loo")) {
   colnames(ic_diffs) <- c(toupper(ic), "SE")
   # compare all models at once to obtain weights
   all_compare <- do.call(loo::compare, x)
-  if (n_models == 2) {
+  if (n_models == 2L) {
     # weights are named differently when comparing only 2 models
     weights <- unname(all_compare[c("weight1", "weight2")])
   } else {
@@ -561,7 +641,7 @@ match_response <- function(models) {
     # checks if all relevant parts of the response are the same 
     # Args:
     #   x, y: named lists as returned by standata
-    to_match <- c("Y", "se", "weights", "cens", "trunc")
+    to_match <- c("Y", "se", "weights", "cens", "trunc", "disp")
     all(ulapply(to_match, function(v) {
       a <- if (is.null(attr(x, "old_order"))) as.vector(x[[v]])
            else as.vector(x[[v]])[attr(x, "old_order")]
@@ -587,7 +667,9 @@ find_names <- function(x) {
   # Args:
   #   x: a character string
   # Notes:
-  #   currently used in hypothesis.brmsfit
+  #   does not use the R parser itself to allow for 
+  #   square brackets and kommas at the end of names
+  #   currently only used in hypothesis.brmsfit
   # Returns:
   #   all valid variable names within the string
   if (!is.character(x) || length(x) > 1) 
@@ -653,7 +735,7 @@ make_point_frame <- function(mf, effects, conditions, groups, family) {
   #   family: the model family
   # Returns:
   #   a data.frame containing the data points to be plotted
-  points <- mf[, effects[1], drop = FALSE]
+  points <- mf[, effects, drop = FALSE]
   points$.RESP <- model.response(mf)
   # get required variables i.e. (grouping) factors
   list_mf <- lapply(as.list(mf), function(x)
@@ -672,13 +754,13 @@ make_point_frame <- function(mf, effects, conditions, groups, family) {
     points <- replicate(nrow(conditions), points, simplify = FALSE)
     for (i in seq_along(points)) {
       cond <- conditions[i, , drop = FALSE]
-      not_na <- c(!is.na(cond))
-      # do it like base::duplicated
-      if (any(not_na)) {
+      not_na <- which(!(is.na(cond) | cond == ".ZERO"))
+      if (length(not_na)) {
+        # do it like base::duplicated
         K <- do.call("paste", c(mf[, not_na, drop = FALSE], sep = "\r")) %in% 
              do.call("paste", c(cond[, not_na, drop = FALSE], sep = "\r"))
       } else {
-        K <- 1:nrow(mf)
+        K <- seq_len(nrow(mf))
       }
       points[[i]]$MargCond[K] <- rownames(conditions)[i] 
     }
@@ -704,13 +786,8 @@ add_samples <- function(x, newpar, dim = numeric(0), dist = "norm", ...) {
   #           a single character vector
   #   dim: dimension of the new parameter
   # Returns:
-  #   a brmsfit object with new (standard normal) samples
-  if (!is(x, "brmsfit")) {
-    stop("x must be of class brmsfit")
-  }
-  if (!identical(dim, numeric(0))) {
-    stop("currently dim must be numeric(0)")
-  }
+  #   a brmsfit object with samples of a new parameter
+  stopifnot(is(x, "brmsfit"), identical(dim, numeric(0)))
   for (i in seq_along(x$fit@sim$samples)) {
     x$fit@sim$samples[[i]][[newpar]] <- 
       do.call(paste0("r", dist), list(x$fit@sim$iter, ...))
