@@ -10,11 +10,7 @@ extract_draws <- function(x, newdata = NULL, re_formula = NULL,
   #   A named list to be understood by linear_predictor.
   #   For non-linear models, every element of draws$nonlinear
   #   can itself be passed to linear_predictor
-  new_ranef <- check_re_formula(re_formula, old_ranef = x$ranef, 
-                                data = x$data)
-  new_formula <- update_re_terms(x$formula, re_formula = re_formula)
-  ee <- extract_effects(new_formula, family = family(x),
-                        nonlinear = x$nonlinear)
+  ee <- extract_effects(formula(x), family = family(x))
   if (is.null(subset) && !is.null(nsamples)) {
     subset <- sample(Nsamples(x), nsamples)
   }
@@ -24,18 +20,71 @@ extract_draws <- function(x, newdata = NULL, re_formula = NULL,
   draws <- nlist(f = prepare_family(x), data = standata, 
                  nsamples = nsamples, autocor = x$autocor)
   
+  # extract draws of auxiliary parameters
+  args <- list(x = x, subset = subset)
+  ilinks <- c(sigma = "exp", shape = "exp", nu = "exp", phi = "exp", 
+              kappa = "exp", zi = "inv_logit", hu = "inv_logit") 
+  valid_auxpars <- valid_auxpars(family(x), effects = ee, autocor = x$autocor)
+  for (ap in valid_auxpars) {
+    if (!is.null(ee[[ap]])) {
+      auxfit <- x
+      auxfit$formula <- update.formula(x$formula, rhs(attr(x$formula, ap)))
+      auxfit$ranef <- tidy_ranef(extract_effects(auxfit$formula), 
+                                 data = model.frame(x))
+      auxstandata <- amend_newdata(newdata, fit = auxfit, 
+                                   re_formula = re_formula,
+                                   allow_new_levels = allow_new_levels)
+      draws[[ap]] <- .extract_draws(auxfit, standata = auxstandata, nlpar = ap,
+                                    re_formula = re_formula, subset = subset)
+      rm(auxfit)
+      draws[[ap]][c("f", "data", "nsamples", "ilink")] <- 
+        list(draws$f, auxstandata, draws$nsamples, ilinks[ap])
+    } else {
+      regex <- paste0("^", ap, "($|_)")
+      draws[[ap]] <- do.call(as.matrix, c(args, pars = regex))
+    }
+  }
+  # parameters for multivariate models
+  if (is.linear(family(x)) && length(ee$response) > 1L) {
+    draws[["sigma"]] <- do.call(as.matrix, c(args, pars = "^sigma($|_)"))
+    draws[["rescor"]] <- do.call(as.matrix, c(args, pars = "^rescor_"))
+    draws[["Sigma"]] <- get_cov_matrix(sd = draws$sigma, cor = draws$rescor)$cov
+  }
+  # autocorrelation parameters
+  if (incl_autocor) {
+    if (get_ar(x$autocor)) {
+      draws[["ar"]] <- do.call(as.matrix, c(args, pars = "^ar\\["))
+    }
+    if (get_ma(x$autocor)) {
+      draws[["ma"]] <- do.call(as.matrix, c(args, pars = "^ma\\["))
+    }
+    if (get_arr(x$autocor)) {
+      draws[["arr"]] <- do.call(as.matrix, c(args, pars = "^arr\\["))
+    }
+    if (is(x$autocor, "cor_bsts")) {
+      if (is.null(newdata)) {
+        draws[["loclev"]] <- do.call(as.matrix, c(args, pars = "^loclev\\["))
+      } else {
+        warning(paste("Local level terms are currently ignored when", 
+                      "'newdata' is specified."), call. = FALSE)
+      }
+    }
+  }
+  # samples of the (non-linear) predictor
   nlpars <- names(ee$nonlinear)
   if (length(nlpars)) {
-    for (i in 1:length(nlpars)) {
+    for (i in seq_along(nlpars)) {
       nlfit <- x
-      nlfit$formula <- update(x$formula, rhs(x$nonlinear[[i]]))
-      nlfit$nonlinear <- NULL
-      nlfit$ranef <- gather_ranef(extract_effects(nlfit$formula), data = x$data)
+      nlfit$formula <- update.formula(x$formula, 
+                         rhs(attr(x$formula, "nonlinear")[[i]]))
+      nlfit$ranef <- tidy_ranef(extract_effects(nlfit$formula), 
+                               data = model.frame(x))
       nlstandata <- amend_newdata(newdata, fit = nlfit, re_formula = re_formula, 
                                   allow_new_levels = allow_new_levels)
       draws$nonlinear[[nlpars[i]]] <- 
         .extract_draws(nlfit, standata = nlstandata, nlpar = nlpars[i], 
                        re_formula = re_formula, subset = subset)
+      rm(nlfit)
       draws$nonlinear[[nlpars[i]]][c("f", "data", "nsamples")] <- 
         list(f = draws$f, data = nlstandata, draws$nsamples)
     }
@@ -53,42 +102,31 @@ extract_draws <- function(x, newdata = NULL, re_formula = NULL,
     keep <- !grepl("^(X|Z|J|C)", names(draws$data))
     draws$data <- subset_attr(draws$data, keep)
   } else {
-    draws <- c(draws, .extract_draws(x, standata = standata, subset = subset,
-                                     re_formula = re_formula))
-  }
-  
-  args <- list(x = x, as.matrix = TRUE, subset = subset)
-  if (has_sigma(family(x), se = ee$se, autocor = x$autocor))
-    draws[["sigma"]] <- do.call(posterior_samples, c(args, pars = "^sigma($|_)"))
-  if (family(x)$family == "student") 
-    draws[["nu"]] <- c(do.call(posterior_samples, c(args, pars = "^nu$")))
-  if (family(x)$family %in% c("beta", "zero_inflated_beta"))
-    draws[["phi"]] <- c(do.call(posterior_samples, c(args, pars = "^phi$")))
-  if (has_shape(family(x))) 
-    draws[["shape"]] <- c(do.call(posterior_samples, c(args, pars = "^shape$")))
-  if (is.linear(family(x)) && length(ee$response) > 1L) {
-    draws[["rescor"]] <- do.call(posterior_samples, c(args, pars = "^rescor_"))
-    draws[["Sigma"]] <- get_cov_matrix(sd = draws$sigma, 
-                                       cor = draws$rescor)$cov
-  }
-  if (incl_autocor) {
-    if (get_ar(x$autocor)) {
-      draws[["ar"]] <- do.call(posterior_samples, c(args, pars = "^ar\\["))
-    }
-    if (get_ma(x$autocor)) {
-      draws[["ma"]] <- do.call(posterior_samples, c(args, pars = "^ma\\["))
-    }
-    if (get_arr(x$autocor)) {
-      draws[["arr"]] <- do.call(posterior_samples, c(args, pars = "^arr\\["))
-    }
-    if (is(x$autocor, "cor_bsts")) {
-      if (is.null(newdata)) {
-        draws[["loclev"]] <- do.call(posterior_samples, 
-                                     c(args, pars = "^loclev\\["))
-      } else {
-        warning(paste("Local level terms are currently ignored when", 
-                      "'newdata' is specified."), call. = FALSE)
+    x$formula <- rm_attr(formula(x), auxpars())
+    x$ranef <- tidy_ranef(extract_effects(formula(x)), 
+                            data = model.frame(x))
+    resp <- ee$response
+    if (length(resp) > 1L && !isTRUE(attr(formula(x), "old_mv"))) {
+      # new multivariate models
+      draws[["mv"]] <- named_list(resp)
+      for (r in resp) {
+        mvfit <- x
+        attr(mvfit$formula, "response") <- r
+        mvstandata <- amend_newdata(newdata, fit = mvfit, 
+                                    re_formula = re_formula, 
+                                    allow_new_levels = allow_new_levels)
+        draws[["mv"]][[r]] <- 
+          .extract_draws(mvfit, standata = mvstandata, subset = subset,
+                         re_formula = re_formula, nlpar = r)
+        rm(mvfit)
+        draws[["mv"]][[r]][c("f", "data", "nsamples")] <- 
+          list(draws$f, mvstandata, draws$nsamples)
       }
+    } else {
+      # univariate models
+      draws <- c(draws, 
+        .extract_draws(x, standata = standata, subset = subset,
+                       re_formula = re_formula))  
     }
   }
   draws
@@ -104,76 +142,76 @@ extract_draws <- function(x, newdata = NULL, re_formula = NULL,
   #   nlpar: optional name of a non-linear parameter
   # Returns:
   #   a named list
-  new_ranef <- check_re_formula(re_formula, old_ranef = x$ranef, 
-                                data = x$data)
-  new_formula <- update_re_terms(x$formula, re_formula = re_formula)
+  new_formula <- update_re_terms(formula(x), re_formula = re_formula)
   ee <- extract_effects(new_formula, family = family(x))
-  nlpar <- ifelse(nchar(nlpar), paste0(nlpar, "_"), "")
-  args <- list(x = x, as.matrix = TRUE, subset = subset)
+  nlpar_usc <- ifelse(nchar(nlpar), paste0(nlpar, "_"), "")
+  args <- list(x = x, subset = subset)
   
   draws <- list(old_cat = is.old_categorical(x))
-  if (!is.null(standata$X) && ncol(standata$X) && !draws$old_cat) {
-    b_pars <- paste0("b_", nlpar, colnames(standata$X))
-    draws[["b"]] <- do.call(posterior_samples, 
-                            c(args, list(pars = b_pars, exact = TRUE)))
+  if (!is.null(standata$X) && ncol(standata$X) && draws$old_cat != 1L) {
+    b_pars <- paste0("b_", nlpar_usc, colnames(standata$X))
+    draws[["b"]] <- 
+      do.call(as.matrix, c(args, list(pars = b_pars, exact = TRUE)))
   }
   if (!is.null(standata$Xm) && ncol(standata$Xm)) {
     monef <- colnames(standata$Xm)
     draws[["bm"]] <- draws$simplex <- vector("list", length(monef))
     for (i in 1:length(monef)) {
-      bm_par <- paste0("b_", nlpar, monef[i])
-      draws[["bm"]][[i]] <- do.call(posterior_samples, 
-                              c(args, list(pars = bm_par, exact = TRUE)))
-      simplex_par <- paste0("simplex_", nlpar, monef[i], 
+      bm_par <- paste0("b_", nlpar_usc, monef[i])
+      draws[["bm"]][[i]] <- 
+        do.call(as.matrix, c(args, list(pars = bm_par, exact = TRUE)))
+      simplex_par <- paste0("simplex_", nlpar_usc, monef[i], 
                             "[", 1:standata$Jm[i], "]")
-      draws[["simplex"]][[i]] <- do.call(posterior_samples, 
-                                         c(args, list(pars = simplex_par, 
-                                                      exact = TRUE)))
+      draws[["simplex"]][[i]] <- 
+        do.call(as.matrix, c(args, list(pars = simplex_par, exact = TRUE)))
       
     }
   }
   if (is.ordinal(family(x))) {
-    draws[["Intercept"]] <- do.call(posterior_samples, 
-                                    c(args, list(pars = "^b_Intercept\\[")))
+    draws[["Intercept"]] <- 
+      do.call(as.matrix, c(args, list(pars = "^b_Intercept\\[")))
     if (!is.null(standata$Xp) && ncol(standata$Xp)) {
       cse_pars <- paste0("^b_", colnames(standata$Xp), "\\[")
-      draws[["p"]] <- do.call(posterior_samples, c(args, list(pars = cse_pars)))
+      draws[["p"]] <- do.call(as.matrix, c(args, list(pars = cse_pars)))
     }
-  } else if (draws$old_cat) {
+  } else if (draws$old_cat == 1L) {
     # old categorical models deprecated as of brms > 0.8.0
     if (!is.null(standata$X)) {
-      draws[["p"]] <- do.call(posterior_samples, c(args, list(pars = "^b_")))
+      draws[["p"]] <- do.call(as.matrix, c(args, list(pars = "^b_")))
     }
   }
   # splines
   splines <- rename(get_spline_labels(ee))
   for (i in seq_along(splines)) {
     draws[["Zs"]][[splines[i]]] <- draws$data[[paste0("Zs_", i)]]
-    s_pars <- paste0("^s_", nlpar, splines[i], "\\[")
-    draws[["s"]][[splines[i]]] <- do.call(posterior_samples,
-                                          c(args, list(pars = s_pars)))
+    s_pars <- paste0("^s_", nlpar_usc, splines[i], "\\[")
+    draws[["s"]][[splines[i]]] <- 
+      do.call(as.matrix, c(args, list(pars = s_pars)))
   }
-  # random effects
-  group <- names(new_ranef)
+  # group-level effects
+  usc_nlpar <- usc(usc(nlpar))
+  new_ranef <- tidy_ranef(ee, model.frame(x))
+  groups <- unique(new_ranef$group)
   # requires initialization to assign S4 objects of the Matrix package
-  draws[["Z"]] <- named_list(group)
-  for (i in seq_along(group)) {
+  draws[["Z"]] <- named_list(groups)
+  for (g in groups) {
+    new_r <- new_ranef[new_ranef$group == g, ]
     # create a single RE design matrix for every grouping factor
-    Z <- lapply(which(ee$random$group == group[i]), 
+    Z <- lapply(unique(new_r$gn), 
                 function(k) get(paste0("Z_", k), standata))
     Z <- do.call(cbind, Z)
-    id <- match(group[i], ee$random$group)
-    gf <- get(paste0("J_", id), standata)
-    r_pars <- paste0("^r_", nlpar, group[i], "\\[")
-    r <- do.call(posterior_samples, c(args, list(pars = r_pars)))
+    gf <- get(paste0("J_", new_r$id[1]), standata)
+    r_pars <- paste0("^r_", g, usc_nlpar, "\\[")
+    r <- do.call(as.matrix, c(args, list(pars = r_pars)))
     if (is.null(r)) {
-      stop(paste("Random effects for each level of grouping factor",
-                 group[i], "not found. Please set ranef = TRUE",
+      stop(paste0("Group-level effects for each level of group '",
+                 g, "' not found. Please set ranef = TRUE ",
                  "when calling brm."), call. = FALSE)
     }
     # match columns of Z with corresponding RE estimates
-    n_levels <- ngrps(x)[[group[[i]]]]
-    used_re <- ulapply(new_ranef[[group[i]]], match, x$ranef[[group[i]]])
+    n_levels <- ngrps(x)[[g]]
+    old_r <- x$ranef[x$ranef$group == g, ]
+    used_re <- ulapply(new_r$coef, match, old_r$coef)
     used_re_pars <- ulapply(used_re, function(k) 
       1:n_levels + (k - 1) * n_levels)
     r <- r[, used_re_pars, drop = FALSE]
@@ -206,8 +244,8 @@ extract_draws <- function(x, newdata = NULL, re_formula = NULL,
       r <- r[, take_levels, drop = FALSE]
       Z <- Z[, take_levels, drop = FALSE]
     }
-    draws[["Z"]][[group[i]]] <- Z
-    draws[["r"]][[group[i]]] <- r
+    draws[["Z"]][[g]] <- Z
+    draws[["r"]][[g]] <- r
   }
   draws
 }
