@@ -10,70 +10,82 @@
 #   A vector of length draws$nsamples containing the pointwise 
 #   log-likelihood fo the ith observation 
 loglik_gaussian <- function(i, draws, data = data.frame()) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, method = "logLik", i = i)
-  args <- list(mean = ilink(get_eta(i, draws), draws$f$link), sd = sigma)
+  args <- list(mean = ilink(get_eta(draws, i), draws$f$link), 
+               sd = get_sigma(draws$sigma, data = draws$data, i = i))
   # censor_loglik computes the conventional loglik in case of no censoring 
   out <- censor_loglik(dist = "norm", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pnorm, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pnorm, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_student <- function(i, draws, data = data.frame()) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, method = "logLik", i = i)
-  args <- list(df = draws$nu, mu = ilink(get_eta(i, draws), draws$f$link), 
-               sigma = sigma)
+  args <- list(df = get_auxpar(draws$nu, i = i), 
+               mu = ilink(get_eta(draws, i), draws$f$link), 
+               sigma = get_sigma(draws$sigma, data = draws$data, i = i))
   out <- censor_loglik(dist = "student", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pstudent, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pstudent, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_cauchy <- function(i, draws, data = data.frame()) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, method = "logLik", i = i)
-  args <- list(df = 1, mu = ilink(get_eta(i, draws), draws$f$link), 
-               sigma = sigma)
+  args <- list(df = 1, mu = ilink(get_eta(draws, i), draws$f$link), 
+               sigma = get_sigma(draws$sigma, data = draws$data, i = i))
   out <- censor_loglik(dist = "student", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pstudent, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pstudent, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_lognormal <- function(i, draws, data = data.frame()) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, method = "logLik", i = i)
-  args <- list(meanlog = ilink(get_eta(i, draws), draws$f$link), sdlog = sigma)
+  sigma <- get_sigma(draws$sigma, data = draws$data, i = i)
+  args <- list(meanlog = ilink(get_eta(draws, i), draws$f$link), sdlog = sigma)
   out <- censor_loglik(dist = "lnorm", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = plnorm, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = plnorm, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
-loglik_gaussian_multi <- function(i, draws, data = data.frame()) {
-  nobs <- draws$data$N_trait * draws$data$K_trait
-  obs <- seq(i, nobs, draws$data$N_trait)
-  eta <- get_eta(obs, draws)
+loglik_gaussian_mv <- function(i, draws, data = data.frame()) {
+  if (!is.null(draws$data$N_trait)) {
+    obs <- seq(i, draws$data$N, draws$data$N_trait)
+    mu <- ilink(get_eta(draws, obs), draws$f$link)
+  } else {
+    mu <- ilink(get_eta(draws, i), draws$f$link)[, 1, ]
+  }
   out <- sapply(1:draws$nsamples, function(s) 
     dmulti_normal(draws$data$Y[i, ], Sigma = draws$Sigma[s, , ], 
-                  mu = ilink(eta[s, ], draws$f$link), log = TRUE))
+                  mu = mu[s, ], log = TRUE))
   # no truncation allowed
   weight_loglik(out, i = i, data = draws$data)
 }
 
-loglik_student_multi <- function(i, draws, data = data.frame()) {
-  nobs <- draws$data$N_trait * draws$data$K_trait
-  obs <- seq(i, nobs, draws$data$N_trait)
-  eta <- get_eta(obs, draws)
+loglik_student_mv <- function(i, draws, data = data.frame()) {
+  if (!is.null(draws$data$N_trait)) {
+    obs <- seq(i, draws$data$N, draws$data$N_trait)
+    mu <- ilink(get_eta(draws, obs), draws$f$link)
+  } else {
+    mu <- ilink(get_eta(draws, i), draws$f$link)[, 1, ]
+  }
+  nu <- get_auxpar(draws$nu, obs)
   out <- sapply(1:draws$nsamples, function(s) 
-    dmulti_student(draws$data$Y[i, ], df = draws$nu[s, ], 
-                   Sigma = draws$Sigma[s, , ], log = TRUE,
-                   mu = ilink(eta[s, ], draws$f$link)))
+    dmulti_student(draws$data$Y[i, ], df = nu[s, ], mu = mu[s, ],
+                   Sigma = draws$Sigma[s, , ], log = TRUE))
   # no truncation allowed
   weight_loglik(out, i = i, data = draws$data)
 }
 
-loglik_cauchy_multi <- function(i, draws, data = data.frame()) {
-  nobs <- draws$data$N_trait * draws$data$K_trait
-  obs <- seq(i, nobs, draws$data$N_trait)
-  eta <- get_eta(obs, draws)
+loglik_cauchy_mv <- function(i, draws, data = data.frame()) {
+  if (!is.null(draws$data$N_trait)) {
+    obs <- seq(i, draws$data$N, draws$data$N_trait)
+    mu <- ilink(get_eta(draws, obs), draws$f$link)
+  } else {
+    mu <- ilink(get_eta(draws, i), draws$f$link)[, 1, ]
+  }
   out <- sapply(1:draws$nsamples, function(s) 
-    dmulti_student(draws$data$Y[i, ], df = 1,  Sigma = draws$Sigma[s, , ], 
-                   mu = ilink(eta[s, ], draws$f$link), log = TRUE))
+    dmulti_student(draws$data$Y[i, ], df = 1, mu = mu[s, ],
+                   Sigma = draws$Sigma[s, , ], log = TRUE))
   # no truncation allowed
   weight_loglik(out, i = i, data = draws$data)
 }
@@ -81,8 +93,8 @@ loglik_cauchy_multi <- function(i, draws, data = data.frame()) {
 loglik_gaussian_cov <- function(i, draws, data = data.frame()) {
   # currently, only ARMA1 processes are implemented
   obs <- with(draws$data, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
-  args <- list(sigma = draws$sigma, se2 = draws$data$se2[obs], 
-               nrows = length(obs))
+  args <- list(sigma = get_sigma(draws$sigma, data = draws$data, i = obs), 
+               se2 = draws$data$se2[obs], nrows = length(obs))
   if (!is.null(draws$ar) && is.null(draws$ma)) {
     # AR1 process
     args$ar <- draws$ar
@@ -96,10 +108,10 @@ loglik_gaussian_cov <- function(i, draws, data = data.frame()) {
     args[c("ar", "ma")] <- draws[c("ar", "ma")]
     Sigma <- do.call(get_cov_matrix_arma1, args)
   }
-  eta <- get_eta(obs, draws)
+  mu <- ilink(get_eta(draws, obs), draws$f$link)
   out <- sapply(1:draws$nsamples, function(s)
-    dmulti_normal(draws$data$Y[obs], Sigma = Sigma[s, , ], log = TRUE,
-                  mu = ilink(eta[s, ], draws$f$link)))
+    dmulti_normal(draws$data$Y[obs], mu = mu[s, ], 
+                  Sigma = Sigma[s, , ], log = TRUE))
   # weights, truncation and censoring not allowed
   out
 }
@@ -107,8 +119,8 @@ loglik_gaussian_cov <- function(i, draws, data = data.frame()) {
 loglik_student_cov <- function(i, draws, data = data.frame()) {
   # currently, only ARMA1 processes are implemented
   obs <- with(draws$data, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
-  args <- list(sigma = draws$sigma, se2 = draws$data$se2[obs], 
-               nrows = length(obs))
+  args <- list(sigma = get_sigma(draws$sigma, data = draws$data, i = obs), 
+               se2 = draws$data$se2[obs], nrows = length(obs))
   if (!is.null(draws$ar) && is.null(draws$ma)) {
     # AR1 process
     args$ar <- draws$ar
@@ -122,11 +134,11 @@ loglik_student_cov <- function(i, draws, data = data.frame()) {
     args[c("ar", "ma")] <- draws[c("ar", "ma")]
     Sigma <- do.call(get_cov_matrix_arma1, args)
   }
-  eta <- get_eta(obs, draws)
+  mu <- ilink(get_eta(draws, obs), draws$f$link)
+  nu <- get_auxpar(draws$nu, obs)
   out <- sapply(1:draws$nsamples, function(s)
-    dmulti_student(draws$data$Y[obs], df = draws$nu[s, ], 
-                   mu = ilink(eta[s, ], draws$f$link), 
-                   Sigma = Sigma[s, , ], log = TRUE))
+    dmulti_student(draws$data$Y[obs], df = nu[s, ], 
+                   mu = mu[s, ], Sigma = Sigma[s, , ], log = TRUE))
   # weights, truncation and censoring not yet allowed
   out
 }
@@ -138,161 +150,170 @@ loglik_cauchy_cov <- function(i, draws, data = data.frame()) {
 
 loglik_gaussian_fixed <- function(i, draws, data = data.frame()) {
   stopifnot(i == 1)
-  eta <- get_eta(1:nrow(draws$data$V), draws)
+  mu <- ilink(get_eta(draws, 1:nrow(draws$data$V)), draws$f$link)
   ulapply(1:draws$nsamples, function(s) 
-    dmulti_normal(draws$data$Y, Sigma = draws$data$V, log = TRUE,
-                  mu = ilink(eta[s, ], draws$f$link)))
+    dmulti_normal(draws$data$Y, mu = mu[s, ], Sigma = draws$data$V, 
+                  log = TRUE))
 }
 
 loglik_student_fixed <- function(i, draws, data = data.frame()) {
   stopifnot(i == 1)
-  eta <- get_eta(1:nrow(draws$data$V), draws)
+  mu <- ilink(get_eta(draws, 1:nrow(draws$data$V)), draws$f$link)
+  nu <- get_auxpar(draws$nu, 1:nrow(draws$data$V))
   sapply(1:draws$nsamples, function(s) 
-    dmulti_student(draws$data$Y, df = draws$nu[s, ], 
-                   Sigma = draws$data$V, log = TRUE,
-                   mu = ilink(eta[s, ], draws$f$link)))
+    dmulti_student(draws$data$Y, df = nu[s, ], mu = mu[s, ],
+                   Sigma = draws$data$V, log = TRUE))
 }
   
 loglik_cauchy_fixed <- function(i, draws, data = data.frame()) {
   stopifnot(i == 1)
-  eta <- get_eta(1:nrow(draws$data$V), draws)
+  mu <- ilink(get_eta(draws, 1:nrow(draws$data$V)), draws$f$link)
   sapply(1:draws$nsamples, function(s) 
-    dmulti_student(draws$data$Y, df = 1, Sigma = draws$data$V, log = TRUE,
-                   mu = ilink(eta[s, ], draws$f$link)))
+    dmulti_student(draws$data$Y, df = 1, mu = mu[s, ],
+                   Sigma = draws$data$V, log = TRUE))
 }
 
 loglik_binomial <- function(i, draws, data = data.frame()) {
   trials <- ifelse(length(draws$data$max_obs) > 1, 
                    draws$data$max_obs[i], draws$data$max_obs) 
-  args <- list(size = trials, prob = ilink(get_eta(i, draws), draws$f$link))
+  args <- list(size = trials, prob = ilink(get_eta(draws, i), draws$f$link))
   out <- censor_loglik(dist = "binom", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pbinom, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pbinom, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }  
 
 loglik_bernoulli <- function(i, draws, data = data.frame()) {
-  if (!is.null(draws$data$N_trait)) {  # 2PL model
-    eta <- get_eta(i, draws) * exp(get_eta(i + draws$data$N_trait, draws))
-  } else {
-    eta <-  get_eta(i, draws)
-  }
+  eta <-  get_eta(draws, i)
   args <- list(size = 1, prob = ilink(eta, draws$f$link))
-  out <- censor_loglik(dist = "binom", args = args, i = i, data = draws$data)
+  out <- censor_loglik(dist = "binom", args = args, i = i, 
+                       data = draws$data)
   # no truncation allowed
-  out <- weight_loglik(out, i = i, data = draws$data)
-  is_nan <- is.nan(out)
-  if (any(is_nan)) {
-    # for 2PL models NaN may occure for numerical reasons
-    warning(paste("observation", i, "had", length(which(is_nan)), 
-                  "logLik draws that were NaN"))
-    out[is_nan] <- mean(out[!is_nan])
-  }
-  out
+  weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_poisson <- function(i, draws, data = data.frame()) {
-  args <- list(lambda = ilink(get_eta(i, draws), draws$f$link))
+  args <- list(lambda = ilink(get_eta(draws, i), draws$f$link))
   out <- censor_loglik(dist = "pois", args = args, i = i, 
                        data = draws$data)
   out <- truncate_loglik(out, cdf = ppois, args = args, 
-                         data = draws$data)
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_negbinomial <- function(i, draws, data = data.frame()) {
-  shape <- get_shape(draws$shape, data = draws$data, method = "logLik", i = i)
-  args <- list(mu = ilink(get_eta(i, draws), draws$f$link), size = shape)
+  shape <- get_shape(draws$shape, data = draws$data, i = i)
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), size = shape)
   out <- censor_loglik(dist = "nbinom", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pnbinom, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pnbinom, args = args, 
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_geometric <- function(i, draws, data = data.frame()) {
-  args <- list(mu = ilink(get_eta(i, draws), draws$f$link), size = 1)
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), size = 1)
   out <- censor_loglik(dist = "nbinom", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pnbinom, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pnbinom, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_exponential <- function(i, draws, data = data.frame()) {
-  args <- list(rate = 1 / ilink(get_eta(i, draws), draws$f$link))
+  args <- list(rate = 1 / ilink(get_eta(draws, i), draws$f$link))
   out <- censor_loglik(dist = "exp", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pexp, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pexp, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_gamma <- function(i, draws, data = data.frame()) {
-  shape <- get_shape(draws$shape, data = draws$data, method = "logLik", i = i)
+  shape <- get_shape(draws$shape, data = draws$data, i = i)
   args <- list(shape = shape, 
-               scale = ilink(get_eta(i, draws), draws$f$link) / shape)
+               scale = ilink(get_eta(draws, i), draws$f$link) / shape)
   out <- censor_loglik(dist = "gamma", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pgamma, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pgamma, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_weibull <- function(i, draws, data = data.frame()) {
-  shape <- get_shape(draws$shape, data = draws$data, method = "logLik", i = i)
-  args <- list(scale = ilink(get_eta(i, draws) / shape, draws$f$link),
+  shape <- get_shape(draws$shape, data = draws$data, i = i)
+  args <- list(scale = ilink(get_eta(draws, i) / shape, draws$f$link),
                shape = shape)
   out <- censor_loglik(dist = "weibull", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pweibull, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pweibull, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_inverse.gaussian <- function(i, draws, data = data.frame()) {
-  args <- list(mean = ilink(get_eta(i, draws), draws$f$link), 
-               shape = draws$shape)
+  args <- list(mean = ilink(get_eta(draws, i), draws$f$link), 
+               shape = get_shape(draws$shape, data = draws$data, i = i))
   out <- censor_loglik(dist = "invgauss", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pinvgauss, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pinvgauss, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_beta <- function(i, draws, data = data.frame()) {
-  mu <- ilink(get_eta(i, draws), draws$f$link)
-  args <- list(shape1 = mu * draws$phi, shape2 = (1 - mu) * draws$phi)
+  mu <- ilink(get_eta(draws, i), draws$f$link)
+  phi <- get_auxpar(draws$phi, i)
+  args <- list(shape1 = mu * phi, shape2 = (1 - mu) * phi)
   out <- censor_loglik(dist = "beta", args = args, i = i, data = draws$data)
-  out <- truncate_loglik(out, cdf = pbeta, args = args, data = draws$data)
+  out <- truncate_loglik(out, cdf = pbeta, args = args,
+                         i = i, data = draws$data)
+  weight_loglik(out, i = i, data = draws$data)
+}
+
+loglik_von_mises <- function(i, draws, data = data.frame()) {
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), 
+               kappa = get_auxpar(draws$kappa, i = i))
+  out <- censor_loglik(dist = "von_mises", args = args, 
+                       i = i, data = draws$data)
+  out <- truncate_loglik(out, cdf = pvon_mises, args = args,
+                         i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_hurdle_poisson <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(lambda = ilink(get_eta(i, draws), draws$f$link))
+  theta <- get_theta(draws, i, par = "hu")
+  args <- list(lambda = ilink(get_eta(draws, i), draws$f$link))
   out <- hurdle_loglik_discrete(pdf = dpois, theta = theta, 
                                 args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_hurdle_negbinomial <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(mu = ilink(get_eta(i, draws), draws$f$link), 
-               size = draws$shape)
+  theta <- get_theta(draws, i, par = "hu")
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), 
+               size = get_shape(draws$shape, data = draws$data, i = i))
   out <- hurdle_loglik_discrete(pdf = dnbinom, theta = theta, 
                                 args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_hurdle_gamma <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(shape = draws$shape, 
-               scale = ilink(get_eta(i, draws), draws$f$link) / draws$shape)
+  theta <- get_theta(draws, i, par = "hu")
+  shape <- get_shape(draws$shape, data = draws$data, i = i)
+  args <- list(shape = shape, 
+               scale = ilink(get_eta(draws, i), draws$f$link) / shape)
   out <- hurdle_loglik_continuous(pdf = dgamma, theta = theta, 
                                   args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_zero_inflated_poisson <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(lambda = ilink(get_eta(i, draws), draws$f$link))
+  theta <- get_theta(draws, i, par = "zi")
+  args <- list(lambda = ilink(get_eta(draws, i), draws$f$link))
   out <- zero_inflated_loglik(pdf = dpois, theta = theta, 
                               args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_zero_inflated_negbinomial <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(mu = ilink(get_eta(i, draws), draws$f$link), 
-               size = draws$shape)
+  theta <- get_theta(draws, i, par = "zi")
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), 
+               size = get_shape(draws$shape, data = draws$data, i = i))
   out <- zero_inflated_loglik(pdf = dnbinom, theta = theta, 
                               args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
@@ -301,17 +322,18 @@ loglik_zero_inflated_negbinomial <- function(i, draws, data = data.frame()) {
 loglik_zero_inflated_binomial <- function(i, draws, data = data.frame()) {
   trials <- ifelse(length(draws$data$max_obs) > 1, 
                    draws$data$max_obs[i], draws$data$max_obs) 
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  args <- list(size = trials, prob = ilink(get_eta(i, draws), draws$f$link))
+  theta <- get_theta(draws, i, par = "zi")
+  args <- list(size = trials, prob = ilink(get_eta(draws, i), draws$f$link))
   out <- zero_inflated_loglik(pdf = dbinom, theta = theta, 
                               args = args, i = i, data = draws$data)
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_zero_inflated_beta <- function(i, draws, data = data.frame()) {
-  theta <- ilink(get_eta(i + draws$data$N_trait, draws), "logit")
-  mu <- ilink(get_eta(i, draws), draws$f$link)
-  args <- list(shape1 = mu * draws$phi, shape2 = (1 - mu) * draws$phi)
+  theta <- get_theta(draws, i, par = "zi")
+  mu <- ilink(get_eta(draws, i), draws$f$link)
+  phi <- get_auxpar(draws$phi, i)
+  args <- list(shape1 = mu * phi, shape2 = (1 - mu) * phi)
   # zi_beta is technically a hurdle model
   out <- hurdle_loglik_continuous(pdf = dbeta, theta = theta, 
                                   args = args, i = i, data = draws$data)
@@ -322,17 +344,18 @@ loglik_categorical <- function(i, draws, data = data.frame()) {
   ncat <- ifelse(length(draws$data$max_obs) > 1, draws$data$max_obs[i], 
                  draws$data$max_obs) 
   if (draws$f$link == "logit") {
-    p <- cbind(rep(0, draws$nsamples), 
-               get_eta(i, draws, ordinal = TRUE)[, 1, ])
+    p <- cbind(rep(0, draws$nsamples), get_eta(draws, i)[, 1, ])
     out <- p[, draws$data$Y[i]] - log(rowSums(exp(p)))
-  } else stop(paste("Link", draws$f$link, "not supported"))
+  } else {
+    stop(paste("Link", draws$f$link, "not supported"))
+  }
   weight_loglik(out, i = i, data = draws$data)
 }
 
 loglik_cumulative <- function(i, draws, data = data.frame()) {
   ncat <- ifelse(length(draws$data$max_obs) > 1, 
                  draws$data$max_obs[i], draws$data$max_obs)
-  eta <- get_eta(i, draws, ordinal = TRUE)
+  eta <- get_eta(draws, i)
   y <- draws$data$Y[i]
   if (y == 1) { 
     out <- log(ilink(eta[, 1, 1], draws$f$link))
@@ -348,7 +371,7 @@ loglik_cumulative <- function(i, draws, data = data.frame()) {
 loglik_sratio <- function(i, draws, data = data.frame()) {
   ncat <- ifelse(length(draws$data$max_obs) > 1, 
                  draws$data$max_obs[i], draws$data$max_obs)
-  eta <- get_eta(i, draws, ordinal = TRUE)
+  eta <- get_eta(draws, i)
   y <- draws$data$Y[i]
   q <- sapply(1:min(y, ncat - 1), function(k) 
     1 - ilink(eta[, 1, k], draws$f$link))
@@ -367,7 +390,7 @@ loglik_sratio <- function(i, draws, data = data.frame()) {
 loglik_cratio <- function(i, draws, data = data.frame()) {
   ncat <- ifelse(length(draws$data$max_obs) > 1, 
                  draws$data$max_obs[i], draws$data$max_obs)
-  eta <- get_eta(i, draws, ordinal = TRUE)
+  eta <- get_eta(draws, i)
   y <- draws$data$Y[i]
   q <- sapply(1:min(y, ncat-1), function(k) 
     ilink(eta[, 1, k], draws$f$link))
@@ -386,7 +409,7 @@ loglik_cratio <- function(i, draws, data = data.frame()) {
 loglik_acat <- function(i, draws, data = data.frame()) {
   ncat <- ifelse(length(draws$data$max_obs) > 1, 
                  draws$data$max_obs[i], draws$data$max_obs)
-  eta <- get_eta(i, draws, ordinal = TRUE)
+  eta <- get_eta(draws, i)
   y <- draws$data$Y[i]
   if (draws$f$link == "logit") { # more efficient calculation 
     q <- sapply(1:(ncat - 1), function(k) eta[, 1, k])
@@ -434,18 +457,19 @@ censor_loglik <- function(dist, args, i, data) {
   }
 }
 
-truncate_loglik <- function(x, cdf, args, data) {
+truncate_loglik <- function(x, cdf, args, i, data) {
   # adjust logLik in truncated models
   # Args:
-  #  x: vector of loglik values
-  #  cdf: a cumulative distribution function 
-  #  args: arguments passed to cdf
-  #  data: data initially passed to Stan
+  #   x: vector of loglik values
+  #   cdf: a cumulative distribution function 
+  #   args: arguments passed to cdf
+  #   i: observation number
+  #   data: data initally passed to Stan
   # Returns:
   #   vector of loglik values
-  if (!(is.null(data$lb[1]) && is.null(data$ub[1]))) {
-    lb <- ifelse(is.null(data$lb[1]), -Inf, data$lb[1])
-    ub <- ifelse(is.null(data$ub[1]), Inf, data$ub[1])
+  if (!(is.null(data$lb) && is.null(data$ub))) {
+    lb <- if (is.null(data$lb)) -Inf else data$lb[i]
+    ub <- if (is.null(data$ub)) -Inf else data$ub[i]
     x - log(do.call(cdf, c(ub, args)) - do.call(cdf, c(lb, args)))
   } else {
     x
@@ -455,9 +479,9 @@ truncate_loglik <- function(x, cdf, args, data) {
 weight_loglik <- function(x, i, data) {
   # weight loglik values according to defined weights
   # Args:
-  #  x: vector of loglik values
-  #  i: observation number
-  #  data: data initially passed to Stan
+  #   x: vector of loglik values
+  #   i: observation number
+  #   data: data initially passed to Stan
   # Returns:
   #   vector of loglik values
   if ("weights" %in% names(data)) {
