@@ -28,7 +28,7 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
                           prior = NULL, autocor = NULL, nonlinear = NULL, 
                           partial = NULL, cov_ranef = NULL, 
                           sample_prior = FALSE, knots = NULL, 
-                          control = NULL, ...) {
+                          control = list(), ...) {
   # internal control arguments:
   #   is_newdata: is make_standata is called with new data?
   #   not4stan: is make_standata called for use in S3 methods?
@@ -44,9 +44,9 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   if (!(is.null(data) || is.list(data)))
     stop("argument 'data' must be a data.frame or list", call. = FALSE)
   family <- check_family(family)
-  nonlinear <- nonlinear2list(nonlinear) 
-  formula <- update_formula(formula, data = data, family = family, 
+  formula <- update_formula(formula, data = data, family = family,
                             partial = partial, nonlinear = nonlinear)
+  old_mv <- isTRUE(attr(formula, "old_mv"))
   autocor <- check_autocor(autocor)
   is_linear <- is.linear(family)
   is_ordinal <- is.ordinal(family)
@@ -54,8 +54,7 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   is_forked <- is.forked(family)
   is_categorical <- is.categorical(family)
   et <- extract_time(autocor$formula)
-  ee <- extract_effects(formula, family = family, et$all, 
-                        nonlinear = nonlinear)
+  ee <- extract_effects(formula, family = family, et$all)
   prior <- as.prior_frame(prior)
   check_prior_content(prior, family = family, warn = FALSE)
   na_action <- if (is_newdata) na.pass else na.omit
@@ -66,19 +65,9 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   
   # sort data in case of autocorrelation models
   if (has_arma(autocor) || is(autocor, "cor_bsts")) {
-    # amend if zero-inflated and hurdle models ever get 
-    # autocorrelation structures as they are also using 'trait'
-    if (is_forked) {
-      stop("no autocorrelation allowed for this model", call. = FALSE)
-    }
-    if (is_linear && length(ee$response) > 1L) {
-      if (!grepl("^trait$|:trait$|^trait:|:trait:", et$group)) {
-        stop(paste("autocorrelation structures for multiple responses must",
-                   "contain 'trait' as grouping variable"), call. = FALSE)
-      } else {
-        to_order <- rmNULL(list(data[["trait"]], data[[et$group]], 
-                                data[[et$time]]))
-      }
+    if (old_mv) {
+      to_order <- rmNULL(list(data[["trait"]], data[[et$group]], 
+                              data[[et$time]]))
     } else {
       to_order <- rmNULL(list(data[[et$group]], data[[et$time]]))
     }
@@ -106,7 +95,7 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
         stop(paste("family", family$family, "expects response variable", 
                    "of non-negative integers"), call. = FALSE)
       }
-    } else if (family$family == "bernoulli") {
+    } else if (family$family %in% "bernoulli") {
       standata$Y <- as.numeric(as.factor(standata$Y)) - 1
       if (any(!standata$Y %in% c(0,1))) {
         stop(paste("family", family$family, "expects response variable", 
@@ -114,12 +103,17 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
       }
     } else if (family$family %in% c("beta", "zero_inflated_beta")) {
       lower <- if (family$family == "beta") any(standata$Y <= 0)
-      else any(standata$Y < 0)
+               else any(standata$Y < 0)
       upper <- any(standata$Y >= 1)
       if (lower || upper) {
-        stop("beta regression requires responses between 0 and 1", 
+        stop("The beta distribution requires responses between 0 and 1.", 
              call. = FALSE)
       }
+    } else if (family$family %in% "von_mises") {
+      if (any(standata$Y < -pi | standata$Y > pi)) {
+        stop("The von_mises distribution requires ",
+             "responses between -pi and pi.", call. = FALSE)
+      }
     } else if (is_categorical) { 
       standata$Y <- as.numeric(as.factor(standata$Y))
       if (length(unique(standata$Y)) < 2L) {
@@ -150,8 +144,11 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
     }
     standata$Y <- as.array(standata$Y)
   }
+  
   # data for various kinds of effects
-  if (length(nonlinear)) {
+  ranef <- tidy_ranef(ee, data)
+  args_eff <- nlist(data, family, ranef, prior, autocor, knots, not4stan)
+  if (length(ee$nonlinear)) {
     nlpars <- names(ee$nonlinear)
     # matrix of covariates appearing in the non-linear formula
     C <- get_model_matrix(ee$covars, data = data)
@@ -160,31 +157,52 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
            call. = FALSE)
     }
     standata <- c(standata, list(KC = ncol(C), C = C)) 
-    for (i in seq_along(nlpars)) {
-      nle <- ee$nonlinear[[i]]
-      data_fixef <- data_fixef(nle, data = data, family = family, 
-                               nlpar = nlpars[i], knots = knots, 
-                               not4stan = not4stan, G = control$G[[i]])
-      data_monef <- data_monef(nle, data = data, prior = prior, 
-                               Jm = control[[paste0("Jm_", nlpars[i])]],
-                               nlpar = nlpars[i])
-      data_ranef <- data_ranef(nle, data = data, family = family, 
-                               cov_ranef = cov_ranef, nlpar = nlpars[i], 
-                               is_newdata = is_newdata, not4stan = not4stan)
-      standata <- c(standata, data_fixef, data_monef, data_ranef)
+    for (nlp in nlpars) {
+      args_eff_spec <- list(effects = ee$nonlinear[[nlp]], nlpar = nlp,
+                            smooth = control$smooth[[nlp]],
+                            Jm = control$Jm[[nlp]],
+                            rm_intercept = FALSE)
+      data_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
+      standata <- c(standata, data_eff)
     }
   } else {
-    data_fixef <- data_fixef(ee, data = data, family = family, 
-                             autocor = autocor, knots = knots,
-                             not4stan = not4stan, G = control$G)
-    data_monef <- data_monef(ee, data = data, prior = prior, Jm = control$Jm)
-    data_csef <- data_csef(ee, data = data)
-    data_ranef <- data_ranef(ee, data = data, family = family, 
-                             cov_ranef = cov_ranef, not4stan = not4stan,
-                             is_newdata = is_newdata)
-    standata <- c(standata, data_fixef, data_monef, data_csef, data_ranef)
-    standata$offset <- model.offset(data)
+    resp <- ee$response
+    if (length(resp) > 1L && !old_mv) {
+      args_eff_spec <- list(effects = ee, Jm = control$Jm[["mu"]],
+                            smooth = control$smooth[["mu"]])
+      for (r in resp) {
+        data_eff <- do.call(data_effects, 
+                            c(args_eff_spec, args_eff, nlpar = r))
+        standata <- c(standata, data_eff)
+        standata[[paste0("offset_", r)]] <- model.offset(data)
+      }
+      if (is.linear(family)) {
+        standata$nresp <- length(resp) 
+        standata$nrescor <- length(resp) * (length(resp) - 1) / 2 
+      }
+    } else {
+      args_eff_spec <- list(effects = ee, Jm = control$Jm[["mu"]],
+                            smooth = control$smooth[["mu"]])
+      data_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
+      standata <- c(standata, data_eff, data_csef(ee, data = data))
+      standata$offset <- model.offset(data)
+    }
+   
   }
+  # data for predictors of scale / shape parameters
+  for (ap in intersect(auxpars(), names(ee))) {
+    args_eff_spec <- list(effects = ee[[ap]], nlpar = ap,
+                          smooth = control$smooth[[ap]],
+                          Jm = control$Jm[[ap]],
+                          rm_intercept = FALSE)
+    data_aux_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
+    standata <- c(standata, data_aux_eff)
+  }
+  # data for grouping factors separated after group-ID
+  data_group <- data_group(ranef, data, cov_ranef = cov_ranef,
+                           old_levels = control$old_levels)
+  standata <- c(standata, data_group)
+  
   # data for specific families
   if (has_trials(family)) {
     if (!length(ee$trials)) {
@@ -228,55 +246,76 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   } 
   if (family$family == "inverse.gaussian" && check_response) {
     # save as data to reduce computation time in Stan
-    if (is.formula(ee[c("weights", "cens")])) {
+    if (is.formula(ee$weights) || is.formula(ee$cens)) {
       standata$log_Y <- log(standata$Y) 
     } else {
       standata$log_Y <- sum(log(standata$Y))
     }
     standata$sqrt_Y <- sqrt(standata$Y)
-  }  
-  # evaluate even if check_response is FALSE to ensure that N_trait is defined
-  if (is_linear && length(ee$response) > 1L) {
-    standata$Y <- matrix(standata$Y, ncol = length(ee$response))
-    NC_trait <- ncol(standata$Y) * (ncol(standata$Y) - 1L) / 2L
-    standata <- c(standata, list(N_trait = nrow(standata$Y), 
-                                 K_trait = ncol(standata$Y),
-                                 NC_trait = NC_trait)) 
   }
-  if (is_forked) {
-    # the second half of Y is only dummy data
-    # that was put into data to make melt_data work correctly
-    standata$N_trait <- nrow(data) / 2L
-    standata$Y <- as.array(standata$Y[1L:standata$N_trait]) 
-  }
-  if (is_categorical && !isTRUE(control$old_cat)) {
-    ncat1m <- standata$ncat - 1L
-    standata$N_trait <- nrow(data) / ncat1m
-    standata$Y <- as.array(standata$Y[1L:standata$N_trait])
-    standata$J_trait <- as.array(matrix(1L:standata$N, ncol = ncat1m))
+  
+  if (old_mv) {
+    # deprecated as of brms 1.0.0
+    # evaluate even if check_response is FALSE to ensure 
+    # that N_trait is defined
+    if (is_linear && length(ee$response) > 1L) {
+      standata$Y <- matrix(standata$Y, ncol = length(ee$response))
+      NC_trait <- ncol(standata$Y) * (ncol(standata$Y) - 1L) / 2L
+      standata <- c(standata, list(N_trait = nrow(standata$Y), 
+                                   K_trait = ncol(standata$Y),
+                                   NC_trait = NC_trait)) 
+      # for compatibility with the S3 methods of brms >= 1.0.0
+      standata$nresp <- standata$K_trait
+      standata$nrescor <- standata$NC_trait
+    }
+    if (is_forked) {
+      # the second half of Y is only dummy data
+      # that was put into data to make melt_data work correctly
+      standata$N_trait <- nrow(data) / 2L
+      standata$Y <- as.array(standata$Y[1L:standata$N_trait]) 
+    }
+    if (is_categorical && !isTRUE(control$old_cat == 1L)) {
+      ncat1m <- standata$ncat - 1L
+      standata$N_trait <- nrow(data) / ncat1m
+      standata$Y <- as.array(standata$Y[1L:standata$N_trait])
+      standata$J_trait <- as.array(matrix(1L:standata$N, ncol = ncat1m))
+    }
   }
+  
   # data for addition arguments
   if (is.formula(ee$se)) {
     standata <- c(standata, list(se = .addition(formula = ee$se, data = data)))
   }
   if (is.formula(ee$weights)) {
     standata <- c(standata, list(weights = .addition(ee$weights, data = data)))
-    if (is.linear(family) && length(ee$response) > 1 || is_forked) 
+    if (old_mv) {
       standata$weights <- standata$weights[1:standata$N_trait]
+    }
   }
   if (is.formula(ee$disp)) {
     standata <- c(standata, list(disp = .addition(ee$disp, data = data)))
   }
   if (is.formula(ee$cens) && check_response) {
     standata <- c(standata, list(cens = .addition(ee$cens, data = data)))
-    if (is.linear(family) && length(ee$response) > 1L || is_forked)
+    if (old_mv) {
       standata$cens <- standata$cens[1:standata$N_trait]
+    }
   }
   if (is.formula(ee$trunc)) {
-    standata <- c(standata, .addition(ee$trunc))
-    if (check_response && (min(standata$Y) < standata$lb || 
-                           max(standata$Y) > standata$ub)) {
-      stop("Some responses are outside of the truncation boundaries.",
+    standata <- c(standata, .addition(ee$trunc, data = data))
+    if (length(standata$lb) == 1L) {
+      standata$lb <- rep(standata$lb, standata$N)
+    }
+    if (length(standata$ub) == 1L) {
+      standata$ub <- rep(standata$ub, standata$N)
+    }
+    if (length(standata$lb) != standata$N || 
+        length(standata$ub) != standata$N) {
+      stop("Invalid truncation bounds", call. = FALSE)
+    }
+    if (check_response && any(standata$Y < standata$lb | 
+                              standata$Y > standata$ub)) {
+      stop("Some responses are outside of the truncation bounds.",
            call. = FALSE)
     }
   }
@@ -313,13 +352,17 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
       } 
     }
     if (Karr) {
+      if (length(ee$response) > 1L) {
+        stop("ARR structure not yet implemented for multivariate models.",
+             call. = FALSE)
+      }
       # ARR effects (autoregressive effects of the response)
       standata$Yarr <- arr_design_matrix(Y = standata$Y, r = Karr, 
                                          group = tgroup)
       standata$Karr <- Karr
     }
   } 
-  if (is(autocor, "cor_fixed")) {
+  if (is(autocor, "cov_fixed")) {
     V <- autocor$V
     rmd_rows <- attr(data, "na.action")
     if (!is.null(rmd_rows)) {
@@ -334,6 +377,10 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
     standata$V <- V
   }
   if (is(autocor, "cor_bsts")) {
+    if (length(ee$response) > 1L) {
+      stop("BSTS structure not yet implemented for multivariate models.",
+           call. = FALSE)
+    }
     if (nchar(et$group)) {
       tgroup <- data[[et$group]]
     } else {
