@@ -1,199 +1,183 @@
-stan_llh <- function(family, se = FALSE, weights = FALSE, trials = FALSE, 
-                     cens = FALSE, disp = FALSE, trunc = .trunc(), 
-                     autocor = cor_arma(), cse = FALSE, nresp = 1L) {
-  # Likelihoods in stan language
-  #
+stan_llh <- function(family, effects = list(), autocor = cor_arma(),
+                     trunc_bounds = NULL) {
+  # Likelihood in Stan language
   # Args:
   #   family: the model family
-  #   se: logical; user defined SEs present?
-  #   weights: logical; weights present?
-  #   trials: logical; number of bernoulli trials given per observation?
-  #   cens: logical; censored data?
-  #   trunc: list containing lower and upper truncation boundaries
-  #   autocor: autocorrelation structure; an object of classe cor_arma
-  #   cse: a flag indicating if category specific effects are present
-  #        (for ordinal models only)
-  #   nresp: number of response variables
-  #
-  # Returns:
-  #   a string containing the likelihood of the model in stan language
+  #   effects: output of extract_effects
+  #   autocor: object of classe cor_brms
   stopifnot(is(family, "family"))
   link <- family$link
-  type <- family$type
   family <- family$family
   is_categorical <- is.categorical(family)
   is_ordinal <- is.ordinal(family)
   is_hurdle <- is.hurdle(family)
   is_zero_inflated <- is.zero_inflated(family)
   is_forked <- is.forked(family)
-  is_multi <- is.linear(family) && nresp > 1L
-  is_trunc <- trunc$lb > -Inf || trunc$ub < Inf
-  if (is_multi) {
+  is_mv <- is.linear(family) && length(effects$response) > 1L
+  
+  has_se <- is.formula(effects$se)
+  has_weights <- is.formula(effects$weights)
+  has_cens <- is.formula(effects$cens)
+  has_disp <- is.formula(effects$disp)
+  has_trials <- is.formula(effects$trials)
+  has_cse <- is.formula(effects$cse)
+  has_trunc <- any(trunc_bounds$lb > -Inf) || any(trunc_bounds$ub < Inf)
+  ll_adj <- has_cens || has_weights || has_trunc
+
+  if (is_mv) {
     # prepare for use of a multivariate likelihood
-    family <- paste0(family, "_multi")
-  } else if (use_cov(autocor) && (get_ar(autocor) || get_ma(autocor))) {
-    # ARMA effects have a special formulation
-    # if fitted using a covariance matrix for residuals
-    if (weights || cens || is_trunc) {
+    family <- paste0(family, "_mv")
+  } else if (use_cov(autocor)) {
+    # ARMA effects in covariance matrix formulation
+    if (ll_adj) {
       stop("Invalid addition arguments", call. = FALSE)
     }
     family <- paste0(family, "_cov")
-  } else if (is(autocor, "cor_fixed")) {
-    if (se || weights || cens || is_trunc) {
+  } else if (is(autocor, "cov_fixed")) {
+    if (has_se || ll_adj) {
       stop("Invalid addition arguments", call. = FALSE)
     }
     family <- paste0(family, "_fixed")
   }
   
-  simplify <- !is_trunc && !cens && stan_has_build_in_fun(family, link)
-  n <- ifelse(cens || weights || is_trunc || is_categorical ||
-              is_ordinal || is_hurdle || is_zero_inflated, "[n]", "")
-  ns <- ifelse((se || trials || disp) && (cens || weights || is_trunc) 
-               || (trials && is_zero_inflated), "[n]", "")
-  disp <- ifelse(disp, "disp_", "")
-  sigma <- paste0(ifelse(se, "se", paste0(disp, "sigma")), ns)
-  shape <- paste0(disp, "shape", ns)
-  ordinal_args <- paste0("eta[n], ", if (cse) "etap[n], ", 
-                         "temp_Intercept")
+  auxpars <- intersect(auxpars(), names(effects))
+  reqn <- ll_adj || is_categorical || is_ordinal || 
+          is_hurdle || is_zero_inflated || 
+          any(c("phi", "kappa") %in% auxpars)
+  n <- ifelse(reqn, "[n]", "")
+  # prepare auxiliary parameters
+  disp <- ifelse(has_disp, "disp_", "")
+  reqn_sigma <- (ll_adj && (has_se || has_disp)) || 
+                (reqn && "sigma" %in% auxpars)
+  sigma <- paste0(ifelse(has_se, "se", paste0(disp, "sigma")), 
+                  if (reqn_sigma) "[n]")
+  reqn_shape <- (ll_adj && has_disp) || (reqn && "shape" %in% auxpars)
+  shape <- paste0(disp, "shape", if (reqn_shape) "[n]")
+  nu <- paste0("nu", if (reqn && "nu" %in% auxpars) "[n]")
+  phi <- paste0("phi", if (reqn && "phi" %in% auxpars) "[n]")
+  kappa <- paste0("kappa", if (reqn && "kappa" %in% auxpars) "[n]")
+  zi <- paste0("zi", if ("zi" %in% auxpars) "[n]")
+  hu <- paste0("hu", if ("hu" %in% auxpars) "[n]")
+  .logit <- ifelse(any(c("zi", "hu") %in% auxpars), "_logit", "")
+  reqn_trials <- has_trials && (ll_adj || is_zero_inflated)
+  trials <- ifelse(reqn_trials, "trials[n]", "trials")
+  ordinal_args <- paste("eta[n],", if (has_cse) "etap[n],", "temp_Intercept")
   
   # use inverse link in likelihood statement only 
   # if it does not prevent vectorization 
-  ilink <- ifelse(n == "[n]" && !simplify, stan_ilink(link), "")
-  eta <- paste0("eta", if (!is.null(type)) paste0("_", type))
-  if (n == "[n]") {
-    if (is_hurdle || is_zero_inflated) {
-      eta <- paste0(ilink,"(eta[n]), ", ilink,"(eta[n + N_trait])")
-    } else {
-      eta <- paste0(eta, "[n]")
-      fl <- ifelse(family %in% c("gamma", "exponential"), 
-                   paste0(family,"_",link), family)
-      eta <- switch(fl, paste0(ilink,"(",eta,")"),
-                    gamma_log = paste0(shape, " * exp(-",eta,")"),
-                    gamma_inverse = paste0(shape, " * ", eta),
-                    gamma_identity =  paste0(shape, " / ", eta),
-                    exponential_log = paste0("exp(-",eta,")"),
-                    exponential_inverse = eta,
-                    exponential_identity = paste0("inv(",eta,")"),
-                    weibull = paste0(ilink, "(",eta, " / ", shape,")"))
-    }
-  }
+  simplify <- !has_trunc && !has_cens && stan_has_built_in_fun(family, link)
+  eta <- paste0(ifelse(is_mv, "Eta", "eta"), n)
   
   if (simplify) { 
     llh_pre <- switch(family,
       poisson = c("poisson_log", eta), 
-      negbinomial = c("neg_binomial_2_log", paste0(eta,", ",shape)),
-      geometric = c("neg_binomial_2_log", paste0(eta,", 1")),
+      negbinomial = c("neg_binomial_2_log", paste0(eta, ", ", shape)),
+      geometric = c("neg_binomial_2_log", paste0(eta, ", 1")),
       cumulative = c("ordered_logistic", "eta[n], temp_Intercept"),
-      categorical = c("categorical_logit", "append_row(zero, eta[J_trait[n]])"),
-      binomial = c("binomial_logit", paste0("trials",ns,", ",eta)), 
+      categorical = c("categorical_logit", "append_row(zero, Eta[n])"),
+      binomial = c("binomial_logit", paste0(trials, ", ", eta)), 
       bernoulli = c("bernoulli_logit", eta))
   } else {
     llh_pre <- switch(family,
-      gaussian = c("normal", paste0(eta,", ",sigma)),
+      gaussian = c("normal", paste0(eta, ", ", sigma)),
       gaussian_cov = c("normal_cov", paste0(eta,", se2, N_tg, ", 
                        "begin_tg, end_tg, nobs_tg, res_cov_matrix")),
-      gaussian_multi = c("multi_normal_cholesky", paste0("Eta",n,", LSigma")),
-      gaussian_fixed = c("multi_normal_cholesky", paste0(eta,", LV")),
-      student = c("student_t",  paste0("nu, ",eta,", ",sigma)),
-      student_cov = c("student_t_cov", paste0("nu, ",eta,", se2, N_tg, ", 
+      gaussian_mv = c("multi_normal_cholesky", paste0(eta, ", LSigma")),
+      gaussian_fixed = c("multi_normal_cholesky", paste0(eta, ", LV")),
+      student = c("student_t",  paste0(nu, ", ", eta, ", ", sigma)),
+      student_cov = c("student_t_cov", paste0(nu, ", ", eta, ", se2, N_tg, ", 
                       "begin_tg, end_tg, nobs_tg, res_cov_matrix")),
-      student_multi = c("multi_student_t", paste0("nu, Eta",n,", Sigma")),
-      student_fixed = c("multi_student_t", paste0("nu, ",eta,", V")),
-      cauchy = c("cauchy", paste0(eta,", ", sigma)),
-      cauchy_cov = c("student_t_cov", paste0("1, ",eta,", se2, N_tg, ", 
-                     "begin_tg, end_tg, nobs_tg, res_cov_matrix")),
-      cauchy_multi = c("multi_student_t", paste0("1.0, Eta",n,", Sigma")),
-      cauchy_fixed = c("multi_student_t", paste0("1.0, ",eta,", V")),
-      lognormal = c("lognormal", paste0(eta,", ",sigma)),
+      student_mv = c("multi_student_t", paste0(nu, ", ", eta, ", Sigma")),
+      student_fixed = c("multi_student_t", paste0(nu, ", ", eta, ", V")),
+      lognormal = c("lognormal", paste0(eta, ", ", sigma)),
       poisson = c("poisson", eta),
-      negbinomial = c("neg_binomial_2", paste0(eta,", ",shape)),
-      geometric = c("neg_binomial_2", paste0(eta,", 1")),
-      binomial = c("binomial", paste0("trials",ns,", ",eta)),
+      negbinomial = c("neg_binomial_2", paste0(eta, ", ", shape)),
+      geometric = c("neg_binomial_2", paste0(eta, ", 1")),
+      binomial = c("binomial", paste0(trials, ", ", eta)),
       bernoulli = c("bernoulli", eta),
-      gamma = c("gamma", paste0(shape,", ",eta)), 
+      gamma = c("gamma", paste0(shape, ", ", eta)), 
       exponential = c("exponential", eta),
-      weibull = c("weibull", paste0(shape,", ",eta)), 
-      inverse.gaussian = c(paste0("inv_gaussian", if (!nchar(n)) "_vector"), 
-                           paste0(eta, ", shape, log_Y",n,", sqrt_Y",n)),
-      beta = c("beta", paste0(eta, " * phi, (1 - ", eta, ") * phi")),
+      weibull = c("weibull", paste0(shape,", ", eta)), 
+      inverse.gaussian = c(paste0("inv_gaussian", if (!reqn) "_vector"), 
+                           paste0(eta, ", shape, log_Y", n, ", sqrt_Y", n)),
+      beta = c("beta", paste0(eta, " * ", phi, ", (1 - ", eta, ") * ", phi)),
+      von_mises = c(paste0("von_mises_", ifelse(reqn, "real", "vector")), 
+                           paste0(eta, ", ", kappa)),
       cumulative = c("cumulative", ordinal_args),
       sratio = c("sratio", ordinal_args),
       cratio = c("cratio", ordinal_args),
       acat = c("acat", ordinal_args),
-      hurdle_poisson = c("hurdle_poisson", "eta[n], eta[n + N_trait]"),
-      hurdle_negbinomial = c("hurdle_neg_binomial_2", 
-                             "eta[n], eta[n + N_trait], shape"),
-      hurdle_gamma = c("hurdle_gamma", "shape, eta[n], eta[n + N_trait]"),
-      zero_inflated_poisson = c("zero_inflated_poisson", 
-                                "eta[n], eta[n + N_trait]"),
-      zero_inflated_negbinomial = c("zero_inflated_neg_binomial_2", 
-                                    "eta[n], eta[n + N_trait], shape"),
-      zero_inflated_binomial = c("zero_inflated_binomial", 
-         paste0("trials",ns,", eta[n], eta[n + N_trait]")),
-      zero_inflated_beta = c("zero_inflated_beta", 
-                             "eta[n], eta[n + N_trait], phi"))
+      hurdle_poisson = c(paste0("hurdle_poisson", .logit), 
+                         paste0(eta, ", ", hu)),
+      hurdle_negbinomial = c(paste0("hurdle_neg_binomial", .logit), 
+                             paste0(eta, ", ", hu, ", ", shape)),
+      hurdle_gamma = c(paste0("hurdle_gamma", .logit), 
+                       paste0("shape, ", eta, ", ", hu)),
+      zero_inflated_poisson = c(paste0("zero_inflated_poisson", .logit), 
+                                paste0(eta, ", ", zi)),
+      zero_inflated_negbinomial = 
+        c(paste0("zero_inflated_neg_binomial", .logit),
+          paste0(eta, ", ", zi, ", ", shape)),
+      zero_inflated_binomial = c(paste0("zero_inflated_binomial", .logit), 
+                                 paste0(trials, ", ", eta, ", ", zi)),
+      zero_inflated_beta = c(paste0("zero_inflated_beta", .logit), 
+                             paste0(eta, ", ", zi, ", ", phi)))
   }
   
   # write likelihood code
-  type <- c("cens", "weights")[match(TRUE, c(cens, weights))]
+  type <- c("cens", "weights")[match(TRUE, c(has_cens, has_weights))]
   if (is.na(type)) type <- "general"
   # prepare for possible truncation
   code_trunc <- ""
-  if (is_trunc) {
+  if (has_trunc) {
     if (type %in% c("cens", "weights")) {
-      stop("truncation is not yet possible in censored or weighted models",
-           call. = FALSE)
+      stop("truncation is not yet possible in censored ", 
+           "or weighted models", call. = FALSE)
     } else {
-      lb <- ifelse(trunc$lb > -Inf, "lb", "")
-      ub <- ifelse(trunc$ub < Inf, "ub", "")
-      code_trunc <- paste0(" T[",lb,", ",ub,"]")
+      lb <- ifelse(any(trunc_bounds$lb > -Inf), "lb[n]", "")
+      ub <- ifelse(any(trunc_bounds$ub < Inf), "ub[n]", "")
+      code_trunc <- paste0(" T[", lb, ", ", ub, "]")
     }
   }
-  add_weights <- ifelse(weights, "weights[n] * ", "")
+  weights <- ifelse(has_weights, "weights[n] * ", "")
   .lpdf <- ifelse(use_int(family), "_lpmf", "_lpdf")
   llh <- switch(type, 
     cens = paste0("  // special treatment of censored data \n",
       "      if (cens[n] == 0) ", 
-      ifelse(!weights, paste0("Y[n] ~ ", llh_pre[1], "(", llh_pre[2],"); \n"),
-             paste0("target += ", add_weights, 
-                    llh_pre[1], .lpdf, "(Y[n] | ", llh_pre[2],"); \n")),
+      ifelse(!has_weights, 
+        paste0("Y[n] ~ ", llh_pre[1], "(", llh_pre[2],"); \n"),
+        paste0("target += ", weights, llh_pre[1], .lpdf, 
+               "(Y[n] | ", llh_pre[2],"); \n")),
       "      else { \n",         
-      "        if (cens[n] == 1) target += ", add_weights, 
-      llh_pre[1], "_lccdf(Y[n] | ", llh_pre[2],"); \n",
-      "        else target += ", add_weights, 
-      llh_pre[1], "_lcdf(Y[n] | ", llh_pre[2],"); \n",
+      "        if (cens[n] == 1) target += ",
+      weights, llh_pre[1], "_lccdf(Y[n] | ", llh_pre[2],"); \n",
+      "        else target += ", 
+      weights, llh_pre[1], "_lcdf(Y[n] | ", llh_pre[2],"); \n",
       "      } \n"),
     weights = paste0("  lp_pre[n] = ", llh_pre[1], .lpdf, "(Y[n] | ",
                      llh_pre[2],"); \n"),
     general = paste0("  Y", n, " ~ ", llh_pre[1], "(", llh_pre[2], ")", 
                      code_trunc, "; \n")) 
   # loop over likelihood if it cannot be vectorized
-  trait <- ifelse(is_multi || is_forked || is_categorical, "_trait", "")
-  if (weights || cens || is_trunc || is_ordinal || is_categorical || 
-      is_hurdle || is_zero_inflated) {
-    llh <- paste0("  for (n in 1:N", trait, ") { \n    ", llh, "    } \n")
+  if (reqn) {
+    llh <- paste0("  for (n in 1:N) { \n    ", llh, "    } \n")
   }
   llh
 }
 
-stan_autocor <- function(family, autocor, prior = prior_frame(),
-                      has_se = FALSE, has_disp = FALSE, 
-                      is_multi = FALSE, nonlinear = NULL) {
+stan_autocor <- function(autocor, effects = list(), family = gaussian(),
+                         prior = prior_frame()) {
   # Stan code related to autocorrelation structures
-  # 
   # Args:
-  #   family: the model family
   #   autocor: autocorrelation structure; object of class cor_brms
+  #   effects: output of extract_effects
+  #   family: the model family
   #   prior: a data.frame containing user defined priors 
   #          as returned by check_prior
-  #   has_se: user defined standard errors present?
-  #   is_multi: is the model multivariate?
-  #   nonlinear: optional list of nonlinear formulas
-  #
-  # Returns:
-  #   stan code for computing AR(R)MA effects
   stopifnot(is(family, "family"))
   is_linear <- is.linear(family)
+  resp <- effects$response
+  is_mv <- is_linear && length(resp) > 1L
   link <- stan_link(family$link)
   Kar <- get_ar(autocor)
   Kma <- get_ma(autocor)
@@ -223,12 +207,15 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
     if (use_cov(autocor)) {
       # if the user wants ARMA effects to be estimated using
       # a covariance matrix for residuals
-      err_msg <- "ARMA covariance matrices are not yet allowed"
-      if (is_multi) {
-        stop(paste(err_msg, "in multivariate models."), call. = FALSE)
+      err_msg <- "ARMA covariance matrices are not yet working"
+      if (is_mv) {
+        stop(err_msg, " in multivariate models", call. = FALSE)
       }
-      if (has_disp) {
-        stop(paste(err_msg, "when specifying 'disp'."), call. = FALSE)
+      if (is.formula(effects$disp)) {
+        stop(err_msg, " when specifying 'disp'", call. = FALSE)
+      }
+      if ("sigma" %in% names(effects)) {
+        stop(err_msg, " when predicting 'sigma'")
       }
       out$data <- paste0(out$data, "  #include 'data_arma_cov.stan' \n")
       out$transD <- "  matrix[max(nobs_tg), max(nobs_tg)] res_cov_matrix; \n"
@@ -248,7 +235,7 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
       # defined selfmade functions for the functions block
       if (family$family == "gaussian") {
         out$fun <- paste0(out$fun, "  #include 'fun_normal_cov.stan' \n")
-      } else { # family %in% c("student", "cauchy")
+      } else {  # family == "student"
         out$fun <- paste0(out$fun, "  #include 'fun_student_t_cov.stan' \n")
       }
       if (Kar && !Kma) {
@@ -260,25 +247,33 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
       }
     } else {
       err_msg <- "Please set cov = TRUE in cor_arma / cor_ar / cor_ma"
-      if (has_se) {
-        stop(paste(err_msg, "when specifying 'se'."), call. = FALSE)
+      if (is.formula(effects$se)) {
+        stop(err_msg, " when specifying 'se'", call. = FALSE)
       }
-      if (length(nonlinear)) {
-        stop(paste(err_msg, "for non-linear models."), call. = FALSE)
+      if (length(effects$nonlinear)) {
+        stop(err_msg, " for non-linear models", call. = FALSE)
       }
-      index <- ifelse(is_multi, "m, k", "n")
-      wsp <- ifelse(is_multi, "      ", "    ")
-      out$transD <- paste0("  matrix[N, Karma] E;  // ARMA design matrix \n",
-                           "  vector[N] e;  // residuals \n") 
-      out$transC1 <- "  E = rep_matrix(0.0, N, Karma); \n" 
-      out$transC2 <- paste0(
-        wsp, "// calculation of ARMA effects \n",
-        wsp, "e[n] = ", link, "(Y[", index, "]) - eta[n]", "; \n",
-        wsp, "for (i in 1:Karma) { \n", 
-        wsp, "  if (n + 1 - i > 0 && n < N && tg[n + 1] == tg[n + 1 - i]) { \n",
-        wsp, "     E[n + 1, i] = e[n + 1 - i]; \n",
-        wsp, "  } \n",
-        wsp, "} \n")
+      if (is_mv) {
+        rs <- usc(resp, "prefix")
+        index <- paste0("n, ", seq_along(rs))
+      } else {
+        rs <- ""
+        index <- "n"
+      }
+      out$modelD <- paste0(
+        "  // objects storing residuals \n",
+        collapse("  matrix[N, Karma] E", rs, "; \n",
+                 "  vector[N] e", rs, "; \n"))
+      out$modelC1 <- collapse("  E", rs, " = rep_matrix(0.0, N, Karma); \n")
+      out$modelC2 <- paste0(
+        "    // computation of ARMA effects \n",
+        collapse("    e", rs, "[n] = ", link, "(Y[", index, "])", 
+                 " - eta", rs, "[n]", "; \n"),
+        "    for (i in 1:Karma) { \n", 
+        "      if (n + 1 - i > 0 && n < N && tg[n + 1] == tg[n + 1 - i]) { \n",
+        collapse("        E", rs, "[n + 1, i] = e", rs, "[n + 1 - i]; \n"),
+        "      } \n",
+        "    } \n")
     } 
   }
   if (Karr) {
@@ -292,7 +287,7 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
       "  // autoregressive effects of the response \n")
     out$prior <- paste0(out$prior, stan_prior(class = "arr", prior = prior))
   }
-  if (is(autocor, "cor_fixed")) {
+  if (is(autocor, "cov_fixed")) {
     if (!is_linear) {
       stop(paste("Fixed residual covariance matrices for family", 
                  family$family, "are not yet implemented"), 
@@ -305,14 +300,18 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
     }
   }
   if (is(autocor, "cor_bsts")) {
-    if (is.forked(family) || family$family %in% c("bernoulli", "categorical")) {
+    if (is_mv || family$family %in% c("bernoulli", "categorical")) {
       stop("The bsts structure is not yet implemented for this family.",
            call. = FALSE)
     }
+    if (length(effects$nonlinear)) {
+      stop("The bsts structure is not yet implemented for non-linear models",
+           call. = FALSE)
+    }
     out$data <- "  vector[N] tg;  // indicates independent groups \n"
     out$par <- paste0("  vector[N] loclev;  // local level terms \n",
                       "  real<lower=0> sigmaLL;  // SD of local level terms \n")
-    if (is_linear && !is_multi) {
+    if (is_linear && !is_mv) {
       # this often helps with convergence
       center <- paste0(link, "(Y[", c("1", "n"), "])")
     } else {
@@ -332,15 +331,13 @@ stan_autocor <- function(family, autocor, prior = prior_frame(),
   out
 }
 
-stan_multi <- function(family, response, prior = prior_frame()) {
+stan_mv <- function(family, response, prior = prior_frame()) {
   # some Stan code for multivariate models
-  #
   # Args:
   #   family: model family
   #   response: names of the response variables
   #   prior: a data.frame containing user defined priors 
   #          as returned by check_prior
-  # 
   # Returns: 
   #   list containing Stan code specific for multivariate models
   stopifnot(is(family, "family"))
@@ -348,36 +345,29 @@ stan_multi <- function(family, response, prior = prior_frame()) {
   nresp <- length(response)
   if (nresp > 1L) {
     if (is.linear(family)) {
-      out$data <- "  #include 'data_multi.stan' \n"
+      out$data <- "  #include 'data_mv.stan' \n"
       out$par <- paste0(
         "  // parameters for multivariate linear models \n",
-        "  vector<lower=0>[K_trait] sigma; \n",
-        "  cholesky_factor_corr[K_trait] Lrescor; \n")
-      out$loop <- c(paste0(
-        "  // restructure linear predictor \n",
-        "  for (m in 1:N_trait) { \n",  
-        "    for (k in 1:K_trait) { \n", 
-        "      int n; \n",
-        "      n = (k - 1) * N_trait + m; \n"), 
-        "    } \n  } \n")
+        "  vector<lower=0>[nresp] sigma; \n",
+        "  cholesky_factor_corr[nresp] Lrescor; \n")
       out$prior <- paste0(
         stan_prior(class = "sigma", coef = response, prior = prior),
         stan_prior(class = "Lrescor", prior = prior))
       if (family$family == "gaussian") {
-        out$transD <- "  cholesky_factor_cov[K_trait] LSigma; \n"
+        out$transD <- "  cholesky_factor_cov[nresp] LSigma; \n"
         out$transC1 <- paste0(
           "  // compute cholesky factor of residual covariance matrix \n",
           "  LSigma = diag_pre_multiply(sigma, Lrescor); \n")
-      } else if (family$family %in% c("student", "cauchy")) {
-        out$transD <- "  cov_matrix[K_trait] Sigma; \n"
+      } else if (family$family == "student") {
+        out$transD <- "  cov_matrix[nresp] Sigma; \n"
         out$transC1 <- paste0(
           "  // compute residual covariance matrix \n",
           "  Sigma = multiply_lower_tri_self_transpose(", 
           "diag_pre_multiply(sigma, Lrescor)); \n")
       }
       out$genD <- paste0(
-        "  matrix[K_trait,K_trait] Rescor; \n",
-        "  vector<lower=-1,upper=1>[NC_trait] rescor; \n")
+        "  matrix[nresp, nresp] Rescor; \n",
+        "  vector<lower=-1,upper=1>[nrescor] rescor; \n")
       out$genC <- paste0(
         "  // take only relevant parts of residual correlation matrix \n",
         "  Rescor = multiply_lower_tri_self_transpose(Lrescor); \n",
@@ -517,7 +507,7 @@ stan_categorical <- function(family) {
   stopifnot(is(family, "family"))
   out <- list()
   if (is.categorical(family)) {
-    out$data <- "  #include 'data_categorical.stan' \n" 
+    out$data <- "  int<lower=2> ncat;  // number of categories' \n" 
     out$tdataD <- "  vector[1] zero; \n"
     out$tdataC <- "  zero[1] = 0; \n"
   }
@@ -525,7 +515,7 @@ stan_categorical <- function(family) {
 }
 
 stan_forked <- function(family) {
-  # Stan code for forked models: zero-inflated, hurdle, and 2PL
+  # Stan code for forked models i.e. zero-inflated / hurdle models
   # Args:
   #   family: the model family
   # Returns:
@@ -534,97 +524,93 @@ stan_forked <- function(family) {
   stopifnot(is(family, "family"))
   out <- list()
   if (is.forked(family)) {
-    out$data <- paste0(
-      "  int<lower=1> N_trait;  // number of observation per trait \n",
-      "  ", ifelse(use_real(family), "real", "int"),
-      " Y[N_trait];  // response variable \n")
-    if (is.zero_inflated(family) || is.hurdle(family)) {
-      if (family$family == "zero_inflated_poisson") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_zero_inflated_poisson.stan' \n")
-      } else if (family$family == "zero_inflated_negbinomial") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_zero_inflated_negbinomial.stan' \n")
-      } else if (family$family == "zero_inflated_binomial") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_zero_inflated_binomial.stan' \n")
-      } else if (family$family == "zero_inflated_beta") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_zero_inflated_beta.stan' \n")
-      } else if (family$family == "hurdle_poisson") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_hurdle_poisson.stan' \n")
-      } else if (family$family == "hurdle_negbinomial") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_hurdle_negbinomial.stan' \n")
-      } else if (family$family == "hurdle_gamma") {
-        out$fun <- paste0(out$fun, 
-                          "  #include 'fun_hurdle_gamma.stan' \n")
-      } 
-    } else if (is.2PL(family)) {
-      out$transD <- "  vector[N_trait] eta_2PL;  // 2PL linear predictor \n"
-      out$transC1 <- paste0("  eta_2PL = head(eta, N_trait)", 
-                            " .* exp(tail(eta, N_trait)); \n")
-    }
+    if (family$family == "zero_inflated_poisson") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_zero_inflated_poisson.stan' \n")
+    } else if (family$family == "zero_inflated_negbinomial") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_zero_inflated_negbinomial.stan' \n")
+    } else if (family$family == "zero_inflated_binomial") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_zero_inflated_binomial.stan' \n")
+    } else if (family$family == "zero_inflated_beta") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_zero_inflated_beta.stan' \n")
+    } else if (family$family == "hurdle_poisson") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_hurdle_poisson.stan' \n")
+    } else if (family$family == "hurdle_negbinomial") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_hurdle_negbinomial.stan' \n")
+    } else if (family$family == "hurdle_gamma") {
+      out$fun <- paste0(out$fun, 
+        "  #include 'fun_hurdle_gamma.stan' \n")
+    } 
   }
   out
 }
 
-stan_inv_gaussian <- function(family, weights = FALSE, cens = FALSE, 
-                              trunc = FALSE) {
+stan_inv_gaussian <- function(family, ll_adj = FALSE) {
   # stan code for inverse gaussian models
-  #
   # Args:
   #   family: the model family
-  #   weights: weights present?
-  #   cens: censored data?
-  #   trunc: truncated data?
-  #
+  #   ll_adj: is the model weighted, censored, or truncated?
   # Returns:
   #   a list of character strings defining the stan code
   #   specific for inverse gaussian models
   stopifnot(is(family, "family"))
   out <- list()
   if (family$family == "inverse.gaussian") {
+    out$fun <- "  #include 'fun_inv_gaussian.stan' \n"
     out$data <- paste0(
       "  // quantities for the inverse gaussian distribution \n",
       "  vector[N] sqrt_Y;  // sqrt(Y) \n")
-    if (weights || cens || trunc) {
+    if (ll_adj) {
       out$data <- paste0(out$data, "  vector[N] log_Y;  // log(Y) \n")
-      out$fun <- paste0(out$fun, 
-        "  #include 'fun_inv_gaussian_pw.stan' \n")
     } else {
       out$data <- paste0(out$data, "  real log_Y;  // sum(log(Y)) \n")
-      out$fun <- paste0(out$fun, 
-        "  #include 'fun_inv_gaussian_vector.stan' \n")
-    } 
-    if (cens || trunc) {
-      out$fun <- paste0(out$fun, 
-        "  #include 'fun_inv_gaussian_cdf.stan' \n",
-        "  #include 'fun_inv_gaussian_ccdf.stan' \n")
     }
   }
   out
 }
 
-stan_disp <- function(disp, family = gaussian()) {
+stan_von_mises <- function(family, ...) {
+  # Stan code for von_mises models
+  stopifnot(is(family, "family"))
+  out <- list()
+  if (family$family == "von_mises") {
+    out$fun <- paste0("  #include 'fun_tan_half.stan' \n",
+                      "  #include 'fun_von_mises.stan' \n")
+  }
+  out
+}
+
+stan_disp <- function(effects, family = gaussian()) {
   # stan code for models with addition argument 'disp'
   # Args:
   #   disp: logical; are dispersion factors specified?
   #   family: the model family
   stopifnot(is(family, "family"))
   out <- list()
-  if (disp) {
+  if (is(effects$disp, "formula")) {
     par <- if (has_sigma(family)) "sigma"
            else if (has_shape(family)) "shape"
            else stop("invalid family for addition argument 'disp'")
+    times <- if(is.null(effects[[par]])) " * " else " .* "
     out$data <- "  vector<lower=0>[N] disp;  // dispersion factors \n"
-    out$transD <- paste0("  vector<lower=0>[N] disp_", par, "; \n")
-    out$transC1 <- paste0("  disp_", par, " = ", par, " * disp; \n")
+    out$modelD <- paste0("  vector[N] disp_", par, "; \n")
+    out$modelC1 <- paste0("  disp_", par, " = ", par, times, "disp; \n")
   }
   out
 }
 
+stan_monotonic <- function(x) {
+  # add the monotonic function to Stan's functions block
+  if (grepl("[^[:alnum:]]monotonic\\(", collapse(x))) {
+    "  #include fun_monotonic.stan \n"
+  } else NULL
+}
+
 stan_misc_functions <- function(family = gaussian(), kronecker = FALSE) {
   # stan code for user defined functions
   # Args:
@@ -646,10 +632,9 @@ stan_misc_functions <- function(family = gaussian(), kronecker = FALSE) {
   out
 }
 
-stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "",
-                       prior = prior_frame(), matrix = FALSE, wsp = 2) {
+stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "", 
+                       prior = prior_frame(), wsp = 2, matrix = FALSE) {
   # Define priors for parameters in Stan language
-  # 
   # Args:
   #   class: the parameter class
   #   coef: the coefficients of this class
@@ -660,44 +645,43 @@ stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "",
   #          as returned by check_prior
   #   matrix: logical; corresponds the class to a parameter matrix?
   #   wsp: an integer >= 0 defining the number of spaces 
-  #      in front of the output string
-  # 
+  #        in front of the output string
   # Returns:
   #   A character strings in stan language that defines priors 
   #   for a given class of parameters. If a parameter has has 
   #   no corresponding prior in prior, an empty string is returned.
-  
   # only consider user defined priors related to this class and group
   wsp <- collapse(rep(" ", wsp))
   prior_only <- isTRUE(attr(prior, "prior_only"))
   hs_df <- attr(prior, "hs_df")
   keep <- prior$class == class & 
-          (prior$coef %in% coef | !nchar(prior$coef)) &
-          (prior$group == group | !nchar(prior$group)) & 
-          (prior$nlpar == nlpar | !nchar(prior$nlpar))
+          (prior$coef %in% coef | !nzchar(prior$coef)) &
+          (prior$group == group | !nzchar(prior$group)) & 
+          (prior$nlpar %in% nlpar | !nzchar(prior$nlpar))
   prior <- prior[keep, ]
   if (!nchar(class) && nrow(prior)) {
     # increment_log_prob statements are directly put into the Stan code
     return(collapse(wsp, prior$prior, "; \n"))
   } 
   
-  # get base prior
-  igroup <- which(with(prior, !nchar(coef) & nchar(group) & nchar(prior)))
-  inlpar <- which(with(prior, !nchar(coef) & nchar(nlpar) & nchar(prior)))
-  iclass <- which(with(prior, !nchar(coef) & !nchar(group) & nchar(prior)))
-  if (length(igroup)) {  
-    # if there is a global prior for this group
-    base_prior <- prior[igroup, "prior"]
-  } else if (length(inlpar)) {
-    # if there is a global prior for this non-linear parameter
-    base_prior <- prior[inlpar, "prior"]
-  } else if (length(iclass)) {  
-    # if there is a global prior for this class
-    base_prior <- prior[iclass, "prior"]
-  } else {  
-    # no proper prior for this class
-    base_prior <- ""
-  } 
+  unlpar <- unique(nlpar)
+  if (length(unlpar) > 1L) {
+    # can only happen for SD parameters of the same ID
+    base_prior <- rep(NA, length(unlpar))
+    for (i in seq_along(unlpar)) {
+      nlpar_prior <- prior[prior$nlpar %in% c("", unlpar[i]), ]
+      base_prior[i] <- stan_base_prior(nlpar_prior)
+    }
+    if (length(unique(base_prior)) > 1L) {
+      # define prior for single coefficients manually
+      # as there is not single base_prior anymore
+      take <- match(prior[nzchar(prior$coef), "nlpar"], unlpar)
+      prior[nzchar(prior$coef), "prior"] <- base_prior[take]
+    }
+    base_prior <- base_prior[1]
+  } else {
+    base_prior <- stan_base_prior(prior)
+  }
   
   individual_prior <- function(i, max_index) {
     # individual priors for each parameter of a class
@@ -706,12 +690,15 @@ stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "",
     } else {
       index <- ""
     }
+    if (length(nlpar) > 1L) {
+      prior <- prior[prior$nlpar == nlpar[i], ]
+    }
     uc_prior <- prior$prior[match(coef[i], prior$coef)]
     if (!is.na(uc_prior) & nchar(uc_prior)) { 
       # user defined prior for this parameter
       coef_prior <- uc_prior
     } else { # base prior for this parameter
-      coef_prior <- base_prior  
+      coef_prior <- base_prior 
     }  
     if (nchar(coef_prior) > 0) {  # implies a proper prior
       out <- paste0(wsp, class, index, " ~ ", coef_prior, "; \n")
@@ -725,7 +712,8 @@ stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "",
   class <- paste0(class, suffix)
   if (any(with(prior, nchar(coef) & nchar(prior)))) {
     # generate a prior for each coefficient
-    out <- sapply(1:length(coef), individual_prior, max_index = length(coef))
+    out <- sapply(seq_along(coef), individual_prior, 
+                  max_index = length(coef))
   } else if (nchar(base_prior) > 0) {
     if (matrix) {
       class <- paste0("to_vector(", class, ")")
@@ -751,6 +739,31 @@ stan_prior <- function(class, coef = "", group = "", nlpar = "", suffix = "",
   out
 }
 
+stan_base_prior <- function(prior) {
+  # get base (highest level) prior of all priors
+  # Args:
+  #   prior: a prior.frame
+  stopifnot(length(unique(prior$class)) <= 1L) 
+  igroup <- which(with(prior, !nchar(coef) & nchar(group) & nchar(prior)))
+  inlpar <- which(with(prior, !nchar(coef) & nchar(nlpar) & nchar(prior)))
+  iclass <- which(with(prior, !nchar(coef) & !nchar(group) & nchar(prior)))
+  if (length(igroup)) {  
+    # if there is a global prior for this group
+    base_prior <- prior[igroup, "prior"]
+  } else if (length(inlpar)) {
+    # if there is a global prior for this non-linear parameter
+    base_prior <- prior[inlpar, "prior"]
+  } else if (length(iclass)) {  
+    # if there is a global prior for this class
+    base_prior <- prior[iclass, "prior"]
+  } else {  
+    # no proper prior for this class
+    base_prior <- ""
+  }
+  stopifnot(length(base_prior) == 1L)
+  base_prior
+}
+
 stan_rngprior <- function(sample_prior, prior, par_declars = "",
                           family = gaussian(), hs_df = NULL) {
   # stan code to sample from priors seperately
@@ -782,16 +795,7 @@ stan_rngprior <- function(sample_prior, prior, par_declars = "",
     pars[has_ind] <- ulapply(pars[has_ind], function(par) {
       ind <- regmatches(par, gregexpr("\\[[[:digit:]]+\\]", par))
       ind <- as.numeric(substr(ind, 2, nchar(ind) - 1))
-      if (grepl("^b\\[", par)) {
-        par <- paste0("b_",ind)
-      } else if (grepl("^bp\\[", par)) {
-        par <- paste0("bp_",ind)
-      } else if (grepl("^sigma\\[", par)) {
-        par <- paste0("sigma_",ind)
-      } else if (grepl("^sd_", par)) {
-        par <- gsub("\\[[[:digit:]]+\\]", paste0("_",ind), par)
-      }
-      return(par)
+      gsub("\\[[[:digit:]]+\\]", paste0("_", ind), par)
     })
     
     # special treatment of lkj_corr_cholesky priors
@@ -856,7 +860,8 @@ stan_link <- function(link) {
   switch(link, identity = "", log = "log", inverse = "inv",
          sqrt = "sqrt", "1/mu^2" = "inv_square", logit = "logit", 
          probit = "inv_Phi", probit_approx = "inv_Phi", 
-         cloglog = "cloglog", cauchit = "cauchit")
+         cloglog = "cloglog", cauchit = "cauchit",
+         tan_half = "tan_half")
 }
 
 stan_ilink <- function(link) {
@@ -866,10 +871,20 @@ stan_ilink <- function(link) {
   switch(link, identity = "", log = "exp", inverse = "inv", 
          sqrt = "square", "1/mu^2" = "inv_sqrt", logit = "inv_logit", 
          probit = "Phi", probit_approx = "Phi_approx", 
-         cloglog = "inv_cloglog", cauchit = "inv_cauchit")
+         cloglog = "inv_cloglog", cauchit = "inv_cauchit",
+         tan_half = "inv_tan_half")
 }
 
-stan_has_build_in_fun <- function(family, link) {
+stan_ll_adj <- function(effects, adds = c("weights", "cens", "trunc")) {
+  # checks if certain 'adds' are present so that the LL has to be adjusted
+  # Args:
+  #   effects: output of extract_effects
+  #   adds: vector of addition argument names
+  stopifnot(all(adds %in% c("weights", "cens", "trunc")))
+  any(ulapply(effects[adds], is.formula))
+}
+
+stan_has_built_in_fun <- function(family, link) {
   # indicates if a family-link combination has a build in 
   # function in Stan (such as binomial_logit)
   (family %in% c("binomial", "bernoulli", "cumulative", "categorical")
@@ -879,10 +894,15 @@ stan_has_build_in_fun <- function(family, link) {
 stan_needs_kronecker <- function(ranef, names_cov_ranef) {
   # checks if a model needs the kronecker product
   # Args: 
-  #   ranef: named list returned by gather_ranef
+  #   ranef: named list returned by tidy_ranef
   #   names_cov_ranef: names of the grouping factors that
   #                    have a cov.ranef matrix 
-  any(ulapply(ranef, function(x, names)
-    length(x) > 1 && attr(x, "group") %in% names && attr(x, "cor"),
-    names = names_cov_ranef))
+  ids <- unique(ranef$id)
+  out <- FALSE
+  for (id in ids) {
+    id_ranef <- subset(ranef, id == id)
+    out <- out || nrow(id_ranef) > 1L && id_ranef$cor[1] &&
+      id_ranef$group[1] %in% names_cov_ranef
+  }
+  out
 }
