@@ -35,8 +35,9 @@ test_that("melt_data keeps factor contrasts", {
   expect_equal(attr(newdata$x, "contrasts"), attr(data$x, "contrasts"))
 })
 
-test_that("melt_data returns expected errors", {
-  ee <- extract_effects(y1 ~ x:main, family = hurdle_poisson())
+test_that("melt_data (deprecated) returns expected errors", {
+  ee <- extract_effects(structure(y1 ~ x:main, old_mv = TRUE), 
+                        family = hurdle_poisson())
   data <- data.frame(y1 = rnorm(10), y2 = rnorm(10), x = 1:10)
   expect_error(melt_data(data = NULL, family = hurdle_poisson(), effects = ee),
                "'data' must be a data.frame", fixed = TRUE)
@@ -44,18 +45,20 @@ test_that("melt_data returns expected errors", {
   expect_error(melt_data(data = data, family = hurdle_poisson(), effects = ee),
                "'main' is a reserved variable name", fixed = TRUE)
   data$response <- 1:10 
-  ee <- extract_effects(response ~ x:main, family = hurdle_poisson())
+  ee <- extract_effects(structure(response ~ x:main, old_mv = TRUE), 
+                        family = hurdle_poisson())
   expect_error(melt_data(data = data, family = hurdle_poisson(), effects = ee),
                "'response' is a reserved variable name", fixed = TRUE)
   data$trait <- 1:10 
-  ee <- extract_effects(y ~ 0 + x*trait, family = hurdle_poisson())
+  ee <- extract_effects(structure(y ~ 0 + x*trait, old_mv = TRUE), 
+                        family = hurdle_poisson())
   expect_error(melt_data(data = data, family = hurdle_poisson(), effects = ee),
                "'trait', 'response' is a reserved variable name", fixed = TRUE)
   
-  ee <- extract_effects(cbind(y1, y2) ~ x)
+  ee <- extract_effects(structure(cbind(y1, y2) ~ x, old_mv = TRUE))
   data <- data.frame(y1 = rnorm(10), y2 = rnorm(10), x = 1:10)
   expect_error(melt_data(data = data, family = poisson(), effects = ee),
-               "invalid multivariate model", fixed = TRUE)
+               "Invalid multivariate model", fixed = TRUE)
 })
 
 test_that("combine_groups does the expected", {
@@ -84,7 +87,7 @@ test_that(paste("arr_design_matrix returns correct design",
 
 
 test_that("amend_newdata handles factors correctly", {
-  fit <- rename_pars(brmsfit_example)
+  fit <- brms:::rename_pars(brmsfit_example1)
   fit$data$fac <- factor(sample(1:3, nrow(fit$data), replace = TRUE))
   newdata <- fit$data[1:5, ]
   expect_silent(amend_newdata(newdata, fit))
@@ -96,21 +99,26 @@ test_that("amend_newdata handles factors correctly", {
                "New factor levels are not allowed")
 })
 
-test_that("update_data handles NAs correctly", {
+test_that("update_data handles NAs correctly in old MV models", {
   data <- data.frame(y1 = c(1, NA, 3), y2 = 4:6, x = 10:12, z = NA)
-  ee <- extract_effects(cbind(y1, y2) ~ x, family = "gaussian")
+  ee <- extract_effects(structure(cbind(y1, y2) ~ x, old_mv = TRUE), 
+                        family = "gaussian")
   expect_warning(mf <- update_data(data, family = "gaussian", effects = ee),
                  "NAs were excluded")
   expect_equivalent(mf, data.frame(response = c(1, 3, 4, 6), y1 = c(1, 3, 1, 3), 
                                    y2 = c(4, 6, 4, 6), x = c(10, 12, 10, 12)))
   
-  ee <- extract_effects(y1 ~ x, family = "hurdle_gamma")
-  expect_warning(mf <- update_data(data, family = "hurdle_gamma", effects = ee),
+  ee <- extract_effects(structure(y1 ~ x, old_mv = TRUE), 
+                        family = "hurdle_gamma")
+  expect_warning(mf <- update_data(data, family = "hurdle_gamma", 
+                                   effects = ee),
                  "NAs were excluded")
-  expect_equivalent(mf, data.frame(response = c(1, 3, 1, 3), y1 = c(1, 3, 1, 3),
+  expect_equivalent(mf, data.frame(response = c(1, 3, 1, 3), 
+                                   y1 = c(1, 3, 1, 3),
                                    x = c(10, 12, 10, 12)))
   
-  ee <- extract_effects(y1 ~ x, family = "zero_inflated_poisson")
+  ee <- extract_effects(structure(y1 ~ x, old_mv = TRUE), 
+                        family = "zero_inflated_poisson")
   expect_warning(mf <- update_data(data, family = "zero_inflated_poisson", 
                                    effects = ee), "NAs were excluded")
   expect_equivalent(mf, data.frame(response = c(1, 3, 1, 3), y1 = c(1, 3, 1, 3),
