@@ -1,5 +1,6 @@
 test_that("self-defined Stan functions work correctly", {
-  skip("expose_stan_functions doesn't work within R CMD CHECK")
+  # for some reason expose_stan_functions doesn't work within R CMD CHECK
+  skip_if_not(exists("new_stan_functions", asNamespace("brms")))
   rstan::expose_stan_functions(brms:::new_stan_functions)
   
   # ARMA matrix generating functions
@@ -35,9 +36,9 @@ test_that("self-defined Stan functions work correctly", {
   shape <- rgamma(1, 20, 1)
   mu <- 20
   y <- statmod::rinvgauss(1, mean = mu, shape = shape)
-  expect_equal(inv_gaussian_cdf_lpdf(y, mu, shape, log(y), sqrt(y)),
+  expect_equal(inv_gaussian_lcdf(y, mu, shape, log(y), sqrt(y)),
                pinvgauss(y, mean = mu, shape = shape, log = TRUE))
-  expect_equal(inv_gaussian_ccdf_lpdf(y, mu, shape, log(y), sqrt(y)),
+  expect_equal(inv_gaussian_lccdf(y, mu, shape, log(y), sqrt(y)),
                log(1 - pinvgauss(y, mean = mu, shape = shape))) 
   expect_equal(inv_gaussian_lpdf(y, mu, shape, log(y), sqrt(y)),
                dinvgauss(y, mean = mu, shape = shape, log = TRUE))
@@ -51,35 +52,67 @@ test_that("self-defined Stan functions work correctly", {
   draws$data <- list(Y = c(0, 10), N_trait = 2, max_obs = 15)
   draws2$data <- list(Y = c(0, 0.5), N_trait = 2)
   for (i in seq_along(draws$data$Y)) {
-    zi_args <- list(y = draws$data$Y[i], eta = draws$eta[i], 
-                    eta_zi = draws$eta[i+2])
-    hu_args <- list(y = draws$data$Y[i], eta = draws$eta[i], 
+    eta_zi_args <- list(y = draws$data$Y[i], eta = draws$eta[i], 
+                        eta_zi = draws$eta[i+2])
+    zi_args <- list(y = draws$data$Y[i], eta = draws$eta[i],
+                    zi = inv_logit(eta_zi_args$eta_zi))
+    eta_hu_args <- list(y = draws$data$Y[i], eta = draws$eta[i], 
                     eta_hu = draws$eta[i+2])
+    hu_args <- list(y = draws$data$Y[i], eta = draws$eta[i],
+                    hu = inv_logit(eta_hu_args$eta_hu))
     draws$f$link <- "log"
+    
     expect_equal(do.call(zero_inflated_poisson_lpmf, zi_args),
                  loglik_zero_inflated_poisson(i, draws))
-    expect_equal(do.call(zero_inflated_neg_binomial_2_lpmf, 
+    expect_equal(do.call(zero_inflated_poisson_logit_lpmf, eta_zi_args),
+                 loglik_zero_inflated_poisson(i, draws))
+    
+    expect_equal(do.call(zero_inflated_neg_binomial_lpmf, 
                          c(zi_args, shape = draws$shape)),
                  loglik_zero_inflated_negbinomial(i, draws))
+    expect_equal(do.call(zero_inflated_neg_binomial_logit_lpmf, 
+                         c(eta_zi_args, shape = draws$shape)),
+                 loglik_zero_inflated_negbinomial(i, draws))
+    
     expect_equal(do.call(hurdle_poisson_lpmf, hu_args),
                  loglik_hurdle_poisson(i, draws))
-    expect_equal(do.call(hurdle_neg_binomial_2_lpmf, 
+    expect_equal(do.call(hurdle_poisson_logit_lpmf, eta_hu_args),
+                 loglik_hurdle_poisson(i, draws))
+    
+    expect_equal(do.call(hurdle_neg_binomial_lpmf, 
                          c(hu_args, shape = draws$shape)),
                  loglik_hurdle_negbinomial(i, draws))
+    expect_equal(do.call(hurdle_neg_binomial_logit_lpmf, 
+                         c(eta_hu_args, shape = draws$shape)),
+                 loglik_hurdle_negbinomial(i, draws))
+    
     expect_equal(do.call(hurdle_gamma_lpdf, 
                          c(hu_args, shape = draws$shape)),
                  loglik_hurdle_gamma(i, draws))
+    expect_equal(do.call(hurdle_gamma_logit_lpdf, 
+                         c(eta_hu_args, shape = draws$shape)),
+                 loglik_hurdle_gamma(i, draws))
+    
     draws$f$link <- "logit"
     expect_equal(do.call(zero_inflated_binomial_lpmf, 
                          c(zi_args, trials = draws$data$max_obs)),
                  loglik_zero_inflated_binomial(i, draws))
+    expect_equal(do.call(zero_inflated_binomial_logit_lpmf, 
+                         c(eta_zi_args, trials = draws$data$max_obs)),
+                 loglik_zero_inflated_binomial(i, draws))
+    
     # zero_inflated_beta requires Y to be in (0,1)
     draws2$f$link <- "logit"
-    zi_args <- list(y = draws2$data$Y[i], eta = draws$eta[i], 
-                    eta_zi = draws$eta[i+2])
+    eta_zi_args <- list(y = draws2$data$Y[i], eta = draws$eta[i], 
+                        eta_zi = draws$eta[i+2])
+    zi_args <- list(y = draws2$data$Y[i], eta = draws$eta[i],
+                    zi = inv_logit(eta_zi_args$eta_zi))
     expect_equal(do.call(zero_inflated_beta_lpdf, 
                          c(zi_args, phi = draws$phi)),
                  loglik_zero_inflated_beta(i, draws2))
+    expect_equal(do.call(zero_inflated_beta_logit_lpdf, 
+                         c(eta_zi_args, phi = draws$phi)),
+                 loglik_zero_inflated_beta(i, draws2))
   }
   
   # ordinal log-densities
@@ -113,7 +146,8 @@ test_that("self-defined Stan functions work correctly", {
                kronecker(t(chol(A)), diag(sd) %*% t(chol(B))))
   
   # as_matrix
-  expect_equal(as_matrix(1:28, 4, 7), rbind(1:7, 8:14, 15:21, 22:28))
+  expect_equal(as_matrix(1:28, 4, 7), 
+               rbind(1:7, 8:14, 15:21, 22:28))
   expect_equal(as_matrix(1:28, 3, 4), rbind(1:4, 5:8, 9:12))
   
   # cauchit and cloglog link
@@ -122,7 +156,10 @@ test_that("self-defined Stan functions work correctly", {
   expect_equal(cloglog(0.2), link(0.2, "cloglog"))
   
   # monotonic
-  expect_equal(monotononic(1:10, 4), sum(1:4))
-  expect_equal(monotononic(rnorm(5), 0), 0)
+  # slightly arkward way to call this function to make sure
+  # is doesn't conflict with the brms R function of the same name
+  monotonic_temp <- get("monotonic", globalenv())
+  expect_equal(monotonic_temp(1:10, 4), sum(1:4))
+  expect_equal(monotonic_temp(rnorm(5), 0), 0)
 })
 
